@@ -1,5 +1,6 @@
-#include <uglib.h>
+#include <UgApp-base.h>
 #include <UgMarkup.h>
+#include <UgData-download.h>
 
 void test_conf ()
 {
@@ -11,9 +12,9 @@ void test_conf ()
 
 	markup		= ug_markup_new ();
 	dataset		= ug_dataset_new ();
-	progress	= ug_dataset_realloc (dataset, UgProgressClass, 0);
-	http		= ug_dataset_realloc (dataset, UgDataHttpClass, 0);
-	ftp			= ug_dataset_realloc (dataset, UgDataFtpClass, 0);
+	progress	= ug_dataset_realloc (dataset, UG_PROGRESS_I, 0);
+	http		= ug_dataset_realloc (dataset, UG_DATA_HTTP_I, 0);
+	ftp			= ug_dataset_realloc (dataset, UG_DATA_FTP_I, 0);
 
 	progress->total = 1484889;
 	http->redirection_limit = 10;
@@ -22,16 +23,16 @@ void test_conf ()
 	ftp->password	= g_strdup ("ftppassword");
 
 	ug_markup_write_start (markup, "test.ug.xml", TRUE);
-	ug_data_to_markup ((UgData*) dataset, markup);
+	ug_data_write_markup ((UgData*) dataset, markup);
 	ug_markup_write_end (markup);
 
 	ug_dataset_unref (dataset);
 
 	dataset = ug_dataset_new ();
 	if (ug_markup_parse ("test.ug.xml", &ug_data_parser, dataset)) {
-		progress	= ug_dataset_realloc (dataset, UgProgressClass, 0);
-		http		= ug_dataset_realloc (dataset, UgDataHttpClass, 0);
-		ftp			= ug_dataset_realloc (dataset, UgDataFtpClass, 0);
+		progress	= ug_dataset_realloc (dataset, UG_PROGRESS_I, 0);
+		http		= ug_dataset_realloc (dataset, UG_DATA_HTTP_I, 0);
+		ftp			= ug_dataset_realloc (dataset, UG_DATA_FTP_I, 0);
 	}
 
 	ug_dataset_unref (dataset);
@@ -40,11 +41,11 @@ void test_conf ()
 
 int main (int argc, char* argv[])
 {
-	uget_class_init ();
+	uglib_init ();
 
 	test_conf ();
 
-	uget_class_finalize ();
+	uglib_finalize ();
 
 	return 0;
 }
