@@ -1,7 +1,7 @@
 /*
  *
- *   Copyright (C) 2005-2010 by Raymond Huang
- *   plushuang at users.sourceforge.net
+ *   Copyright (C) 2005-2011 by C.H. Huang
+ *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -37,55 +37,49 @@
 #include <stdlib.h>
 #include <string.h>
 // uglib
+#include <UgString.h>
 #include <UgData-download.h>
 #include <UgCategory-gtk.h>
 
 #include <glib/gi18n.h>
 
+// UgCategory::user.category used for getting UgCategoruGtk
+// UgRelation::user.category used for getting UgCategoruGtk
+// UgRelation::user.storage  used for getting GtkListStore in primary UgCategoruGtk
+// UgRelation::user.position used for getting GtkTreeIter from GtkListStore in primary UgCategoruGtk
+// UgRelation::user.data     used for getting UgDownloadWidget
+
 // static functions
-static void		ug_category_gtk_init     (UgCategoryGtk* cgtk);
-static void		ug_category_gtk_finalize (UgCategoryGtk* cgtk);
-static gboolean	ug_category_gtk_watch (UgCategoryGtk* ccmd, UgDataset* dataset, UgMessage* message, gpointer user_data);
-// Function used by GtkTreeModelSort.
-static gint		ug_download_model_compare (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer user_data);
+static UgCategoryGtk*	ug_category_gtk_new  (void);
+static void				ug_category_gtk_free (UgCategoryGtk* cgtk);
+
+// free GtkTreeIter in UgRelation::position
+static void		ug_slice_free_tree_iter (GtkTreeIter* iter);
 // Function used by GtkTreeModelFilter.
-static gboolean	ug_download_model_filter_category (GtkTreeModel *model, GtkTreeIter *iter, UgCategoryGtk* category);
+static gboolean	ug_download_model_filter_category (GtkTreeModel *model, GtkTreeIter *iter, UgCategory* category);
+static gboolean	ug_download_model_filter_active   (GtkTreeModel *model, GtkTreeIter *iter, gpointer data);
 static gboolean	ug_download_model_filter_queuing  (GtkTreeModel *model, GtkTreeIter *iter, gpointer data);
 static gboolean	ug_download_model_filter_finished (GtkTreeModel *model, GtkTreeIter *iter, gpointer data);
 static gboolean	ug_download_model_filter_recycled (GtkTreeModel *model, GtkTreeIter *iter, gpointer data);
 
-// define UgDataEntry for UgDataClass
-static UgDataEntry	category_gtk_data_entry[] =
+const UgCategoryFuncs	cgtk_funcs =
 {
-	UG_CATEGORY_DATA_ENTRY,
-	{NULL},			// null-terminated
+	ug_category_gtk_add,
+	ug_category_gtk_get_all,
+	ug_category_gtk_get_tasks,
+	ug_category_gtk_changed,
 };
 
-static UgDataClass	category_gtk_data_class =
-{
-	UG_CATEGORY_CLASS_NAME,		// name
-	NULL,						// reserve
-	sizeof (UgCategoryGtk),		// instance_size
-	category_gtk_data_entry,	// entry
-
-	(UgInitFunc)     ug_category_gtk_init,
-	(UgFinalizeFunc) ug_category_gtk_finalize,
-	(UgAssignFunc)   ug_category_assign,
-};
-// extern
-const	UgDataClass*	UgCategoryGtkClass = &category_gtk_data_class;
-
-static void ug_category_gtk_init (UgCategoryGtk* cgtk)
+static UgCategoryGtk*	ug_category_gtk_new  (void)
 {
+	UgCategoryGtk*		cgtk;
 	GtkTreeView*		view;
 	GtkTreeViewColumn*	column;
 
-	// UgCategory
-	ug_category_init ((UgCategory*) cgtk);
-	cgtk->watch.func = (UgWatchFunc) ug_category_gtk_watch;
-	// UgCategoryGtk
-	g_queue_init (&cgtk->active);
+	cgtk = g_malloc0 (sizeof (UgCategoryGtk));
+	// initialize UgDownloadWidget
 	ug_download_widget_init (&cgtk->all,      NULL);
+	ug_download_widget_init (&cgtk->active,   NULL);
 	ug_download_widget_init (&cgtk->queuing,  NULL);
 	ug_download_widget_init (&cgtk->finished, NULL);
 	ug_download_widget_init (&cgtk->recycled, NULL);
@@ -103,6 +97,12 @@ static void ug_category_gtk_init (UgCategoryGtk* cgtk)
 	gtk_tree_view_column_set_visible (column, FALSE);
 	column = gtk_tree_view_get_column (view, UG_DOWNLOAD_COLUMN_SPEED);
 	gtk_tree_view_column_set_visible (column, FALSE);
+	column = gtk_tree_view_get_column (view, UG_DOWNLOAD_COLUMN_UPLOAD_SPEED);
+	gtk_tree_view_column_set_visible (column, FALSE);
+	column = gtk_tree_view_get_column (view, UG_DOWNLOAD_COLUMN_UPLOADED);
+	gtk_tree_view_column_set_visible (column, FALSE);
+	column = gtk_tree_view_get_column (view, UG_DOWNLOAD_COLUMN_RATIO);
+	gtk_tree_view_column_set_visible (column, FALSE);
 	// Recycled column
 	view = cgtk->recycled.view;
 	column = gtk_tree_view_get_column (view, UG_DOWNLOAD_COLUMN_ELAPSED);
@@ -111,206 +111,170 @@ static void ug_category_gtk_init (UgCategoryGtk* cgtk)
 	gtk_tree_view_column_set_visible (column, FALSE);
 	column = gtk_tree_view_get_column (view, UG_DOWNLOAD_COLUMN_SPEED);
 	gtk_tree_view_column_set_visible (column, FALSE);
+	column = gtk_tree_view_get_column (view, UG_DOWNLOAD_COLUMN_UPLOAD_SPEED);
+	gtk_tree_view_column_set_visible (column, FALSE);
+	column = gtk_tree_view_get_column (view, UG_DOWNLOAD_COLUMN_UPLOADED);
+	gtk_tree_view_column_set_visible (column, FALSE);
+	column = gtk_tree_view_get_column (view, UG_DOWNLOAD_COLUMN_RATIO);
+	gtk_tree_view_column_set_visible (column, FALSE);
+
+	return cgtk;
 }
 
-static void ug_category_gtk_finalize (UgCategoryGtk* cgtk)
+static void	ug_category_gtk_free (UgCategoryGtk* cgtk)
 {
 	UgDataset*		dataset;
+	UgRelation*		relation;
 	GtkTreeModel*	model;
 	GtkTreeIter		iter;
 
-	// UgCategory
-	ug_category_finalize ((UgCategory*) cgtk);
 	// free UgDownloadWidget
 	ug_download_widget_finalize (&cgtk->all);
+	ug_download_widget_finalize (&cgtk->active);
 	ug_download_widget_finalize (&cgtk->queuing);
 	ug_download_widget_finalize (&cgtk->finished);
 	ug_download_widget_finalize (&cgtk->recycled);
-	// free all jobs
+	// free all tasks
 	model = cgtk->filter;
 	while (gtk_tree_model_get_iter_first (model, &iter)) {
 		gtk_tree_model_get (model, &iter, 0, &dataset, -1);
-		ug_category_gtk_remove (cgtk, dataset);
+		// ug_category_gtk_remove (cgtk, dataset);
+		relation = UG_DATASET_RELATION (dataset);
+		gtk_list_store_remove (
+				relation->user.storage,
+				relation->user.position);
+		relation->user.storage = NULL;
+		relation->category = NULL;
+		// delete data and files
+		ug_download_delete_temp (dataset);
+		ug_dataset_unref (dataset);
 	}
-	// free others
-	if (cgtk->store)	// category->primary == NULL
+
+	// If it is primary category, UgCategoryGtk::store == UgCategoryGtk::filter
+	// Only primary one has UgCategoryGtk::store
+	if (cgtk->store)
 		g_object_unref (cgtk->store);
 	else if (cgtk->filter)
 		g_object_unref (cgtk->filter);
-	g_queue_clear (&cgtk->active);
 }
 
-UgCategoryGtk*	ug_category_gtk_new (UgCategoryGtk* primary)
+// If primary == NULL, this category will be a primary category.
+// One uget-gtk program has only one primary category.
+// All dataset store in primary category.
+// Other categories use filter to classify dataset in primary category.
+UgCategory*		ug_category_new_with_gtk (UgCategory* primary)
 {
-	UgCategoryGtk*		category;
+	UgCategory*		category;
 
-	category = ug_data_new (&category_gtk_data_class);
-	ug_category_gtk_set_primary (category, primary);
+	category = ug_category_new ();
+	ug_category_use_gtk (category, primary);
 	return category;
 }
 
-void	ug_category_gtk_free (UgCategoryGtk* category)
+void	ug_category_use_gtk (UgCategory* category, UgCategory* primary)
 {
-	ug_data_free (category);
-}
+	UgCategoryGtk*	cgtk;
 
-void	ug_category_gtk_set_primary (UgCategoryGtk* category, UgCategoryGtk* primary)
-{
-	// clear
-	if (category->store)	// category->primary == NULL
-		g_object_unref (category->store);
-	else if (category->filter)
-		g_object_unref (category->filter);
-
-	category->primary = primary;
-	if (primary == NULL) {
-		category->store = gtk_list_store_new (1, G_TYPE_POINTER);
-		category->filter = GTK_TREE_MODEL (category->store);
+	cgtk = category->user.category;
+	if (cgtk == NULL) {
+		cgtk = ug_category_gtk_new ();
+		category->user.category = cgtk;
+		// destroy notify
+		category->destroy.func  = (UgNotifyFunc) ug_category_gtk_free;
+		category->destroy.data  = category->user.category;
+		// functions
+		category->funcs = &cgtk_funcs;
 	}
-	else {	// primary == NULL
-		category->store = NULL;
-		category->filter = gtk_tree_model_filter_new (
-				GTK_TREE_MODEL (primary->store), NULL);
-		gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (category->filter),
+
+	// primary related code -------------------------------
+	// If it is primary category, UgCategoryGtk::store == UgCategoryGtk::filter
+	// Only primary one has UgCategoryGtk::store
+	if (cgtk->store)
+		g_object_unref (cgtk->store);
+	else if (cgtk->filter)
+		g_object_unref (cgtk->filter);
+
+	// If primary == NULL, this is primary category
+	if (primary == NULL) {
+		cgtk->primary = NULL;
+		cgtk->store   = gtk_list_store_new (1, G_TYPE_POINTER);
+		cgtk->filter  = GTK_TREE_MODEL (cgtk->store);
+	}
+	else {
+		cgtk->primary = primary->user.category;
+		cgtk->store   = NULL;
+		cgtk->filter  = gtk_tree_model_filter_new (
+				GTK_TREE_MODEL (cgtk->primary->store), NULL);
+		gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (cgtk->filter),
 				(GtkTreeModelFilterVisibleFunc) ug_download_model_filter_category,
 				category, NULL);
 	}
 
-	ug_download_widget_set_sortable (&category->all, category->filter,
-			ug_download_model_compare, NULL);
-	ug_download_widget_set_filter (&category->queuing, category->filter,
+	ug_download_widget_use_sortable (&cgtk->all, cgtk->filter);
+	// filter
+	ug_download_widget_set_filter (&cgtk->active, cgtk->all.model,
+			ug_download_model_filter_active, NULL);
+	ug_download_widget_set_filter (&cgtk->queuing, cgtk->filter,
 			ug_download_model_filter_queuing, NULL);
-	ug_download_widget_set_filter (&category->finished, category->filter,
+	ug_download_widget_set_filter (&cgtk->finished, cgtk->filter,
 			ug_download_model_filter_finished, NULL);
-	ug_download_widget_set_filter (&category->recycled, category->filter,
+	ug_download_widget_set_filter (&cgtk->recycled, cgtk->filter,
 			ug_download_model_filter_recycled, NULL);
 }
 
-void	ug_category_gtk_add (UgCategoryGtk* category, UgDataset* dataset)
+void	ug_category_gtk_add (UgCategory* category, UgDataset* dataset)
 {
+	UgCategoryGtk*		cgtk;
 	UgCategoryGtk*		primary;
-	UgRelationGtk*		relation;
+	UgRelation*			relation;
 	GtkTreeModel*		model;
 	GtkTreePath*		path;
 
-	// add and set UgRelationGtkClass to dataset
-	UG_DATASET_RELATION_CLASS (dataset) = UgRelationGtkClass;
-	relation = UG_DATASET_RELATION_GTK (dataset);
+	// add and set UgRelation to dataset
+	relation = UG_DATASET_RELATION (dataset);
 	if (relation == NULL)
-		relation = ug_dataset_alloc_front (dataset, UgRelationGtkClass);
+		relation = ug_dataset_alloc_front (dataset, UG_RELATION_I);
+	// GtkTreeIter in relation->user.position
+	if (relation->user.position == NULL) {
+		relation->user.position = g_slice_alloc (sizeof (GtkTreeIter));
+		relation->destroy.func = (UgNotifyFunc) ug_slice_free_tree_iter;
+		relation->destroy.data = relation->user.position;
+	}
 
-	primary = (category->primary) ? category->primary : category;
-	if (relation->category == NULL  &&  category != primary) {
+	cgtk = category->user.category;
+	primary = (cgtk->primary) ? cgtk->primary : cgtk;
+	// If dataset wasn't added to secondary category
+	if (relation->category == NULL  &&  cgtk != primary) {
 		relation->category = category;
-		if (relation->primary) {
-			model = GTK_TREE_MODEL (relation->primary->store);
-			path = gtk_tree_model_get_path (model, &relation->iter);
-			gtk_tree_model_row_changed (model, path, &relation->iter);
+		// if dataset was added to primary GtkListStore
+		if (relation->user.storage) {
+			model = GTK_TREE_MODEL (relation->user.storage);
+			path = gtk_tree_model_get_path (model, relation->user.position);
+			gtk_tree_model_row_changed (model, path, relation->user.position);
 			gtk_tree_path_free (path);
 		}
 	}
-	if (relation->primary == NULL  &&  primary) {
-		relation->primary = primary;
+	// If dataset wasn't added to primary category
+	if (relation->user.storage == NULL  &&  primary) {
+		// setup relation->user.data for ug_category_gtk_changed().
 		relation->hints &= ~UG_HINT_ACTIVE;
-		gtk_list_store_append (primary->store, &relation->iter);
-		gtk_list_store_set (primary->store, &relation->iter, 0, dataset, -1);
+		if (relation->hints & UG_HINT_RECYCLED)
+			relation->user.data = &cgtk->recycled;
+		else if (relation->hints & UG_HINT_FINISHED)
+			relation->user.data = &cgtk->finished;
+		else
+			relation->user.data = &cgtk->queuing;
+		// add to primary category
+		relation->user.storage = primary->store;
+		gtk_list_store_append (relation->user.storage, relation->user.position);
+		gtk_list_store_set (relation->user.storage, relation->user.position,
+							0, dataset, -1);
+		// reference count
 		ug_dataset_ref (dataset);
 	}
 }
 
-void	ug_category_gtk_remove (UgCategoryGtk* category, UgDataset* dataset)
-{
-	UgRelationGtk*		relation;
-
-	relation = UG_DATASET_RELATION_GTK (dataset);
-	if (relation->category)
-		ug_category_gtk_stop (relation->category, dataset);
-	if (relation->primary)
-		gtk_list_store_remove (relation->primary->store, &relation->iter);
-	relation->primary = NULL;
-	relation->category = NULL;
-	// delete data and files
-	ug_download_temp_delete (dataset);
-	ug_dataset_unref (dataset);
-}
-
-void	ug_category_gtk_switch (UgCategoryGtk* category, UgDataset* dataset, UgCategoryHints hint)
-{
-	UgRelationGtk*		relation;
-	GtkListStore*		store;
-	GtkTreeModel*		model;
-	GtkTreePath*		path;
-
-	relation = UG_DATASET_RELATION_GTK (dataset);
-	store = relation->primary->store;
-
-	if (hint & UG_HINT_RECYCLED) {
-		gtk_list_store_move_after (store, &relation->iter, NULL);
-		relation->hints &= ~UG_HINT_FINISHED;
-		relation->hints |=  UG_HINT_RECYCLED;
-	}
-	else if (hint & UG_HINT_FINISHED) {
-		gtk_list_store_move_after (store, &relation->iter, NULL);
-		relation->hints &= ~UG_HINT_RECYCLED;
-		relation->hints |=  UG_HINT_FINISHED;
-	}
-	else {
-		gtk_list_store_move_before (store, &relation->iter, NULL);
-		relation->hints &= ~(UG_HINT_FINISHED | UG_HINT_RECYCLED);
-	}
-
-	model = GTK_TREE_MODEL (store);
-	path = gtk_tree_model_get_path (model, &relation->iter);
-	gtk_tree_model_row_changed (model, path, &relation->iter);
-	gtk_tree_path_free (path);
-}
-
-void	ug_category_gtk_clear (UgCategoryGtk* category, UgCategoryHints hint, guint from_nth)
-{
-	UgDataset*		dataset;
-	GtkTreeModel*	model;
-	GtkTreeIter		iter;
-	GList*			list;
-	GList*			link;
-	gboolean		valid;
-
-	if (hint & UG_HINT_RECYCLED)
-		model = category->recycled.model;
-	else if (hint & UG_HINT_FINISHED)
-		model = category->finished.model;
-	else
-		model = category->queuing.model;
-
-	// get jobs to clear
-	list = NULL;
-	valid = gtk_tree_model_iter_nth_child (model, &iter, NULL, from_nth);
-	while (valid) {
-		gtk_tree_model_get (model, &iter, 0, &dataset, -1);
-		list = g_list_prepend (list, dataset);
-		valid = gtk_tree_model_iter_next (model, &iter);
-	}
-	// remove jobs
-	for (link = list;  link;  link = link->next)
-		ug_category_gtk_remove (category, link->data);
-	g_list_free (list);
-}
-
-void	ug_category_gtk_move_to (UgCategoryGtk* category, UgDataset* dataset, UgCategoryGtk* category_dest)
-{
-	UgRelationGtk*		relation;
-
-	relation = UG_DATASET_RELATION_GTK (dataset);
-	if (relation->category == category_dest)
-		return;
-
-	// stop active job in secondary category
-	ug_category_gtk_stop (category, dataset);
-	// move
-	relation->category = category_dest;
-	ug_category_gtk_switch (category_dest, dataset, relation->hints);
-}
-
-GList*	ug_category_gtk_get_list (UgCategoryGtk* category)
+GList*	ug_category_gtk_get_all (UgCategory* category)
 {
 	UgDataset*		dataset;
 	GtkTreeModel*	model;
@@ -319,7 +283,7 @@ GList*	ug_category_gtk_get_list (UgCategoryGtk* category)
 	gboolean		valid;
 
 	list = NULL;
-	model = category->filter;
+	model = ((UgCategoryGtk*)category->user.category)->filter;
 	valid = gtk_tree_model_get_iter_first (model, &iter);
 	while (valid) {
 		gtk_tree_model_get (model, &iter, 0, &dataset, -1);
@@ -329,18 +293,168 @@ GList*	ug_category_gtk_get_list (UgCategoryGtk* category)
 	return g_list_reverse (list);
 }
 
-gboolean	ug_category_gtk_move_selected_up (UgCategoryGtk* category, UgDownloadWidget* dwidget)
+GList*	ug_category_gtk_get_tasks (UgCategory* category)
+{
+	UgCategoryGtk*	cgtk;
+	UgDataset*		dataset;
+	UgRelation*		relation;
+	GtkTreeIter		iter;
+	gboolean		valid;
+	GList*			tasks;
+	guint			tasks_len;
+	guint			temp_len;
+
+	cgtk = category->user.category;
+	temp_len = gtk_tree_model_iter_n_children (cgtk->active.model, NULL);
+	if (category->active_limit <= temp_len)
+		return NULL;
+
+	tasks = NULL;
+	tasks_len = 0;
+	temp_len = category->active_limit - temp_len;
+
+	valid = gtk_tree_model_get_iter_first (cgtk->queuing.model, &iter);
+	while (valid) {
+		gtk_tree_model_get (cgtk->queuing.model, &iter, 0, &dataset, -1);
+		valid = gtk_tree_model_iter_next (cgtk->queuing.model, &iter);
+		// check
+		relation = UG_DATASET_RELATION (dataset);
+		if (relation->hints & UG_HINT_UNRUNNABLE)
+			continue;
+		tasks = g_list_prepend (tasks, dataset);
+		tasks_len++;
+		if (tasks_len >= temp_len)
+			break;
+	}
+
+	return g_list_reverse (tasks);
+}
+
+void	ug_category_gtk_changed	(UgCategory* category, UgDataset* dataset)
+{
+	UgDownloadWidget*	dlwidget;
+	UgRelation*			relation;
+	UgCategoryGtk*		cgtk;
+	GtkTreeModel*		model;
+	GtkTreePath*		path;
+
+	cgtk     = category->user.category;
+	relation = UG_DATASET_RELATION (dataset);
+
+	if (relation->hints & UG_HINT_RECYCLED) {
+		dlwidget = &cgtk->recycled;
+		relation->hints &= ~UG_HINT_FINISHED;
+	}
+	else if (relation->hints & UG_HINT_FINISHED) {
+		dlwidget = &cgtk->finished;
+		relation->hints &= ~UG_HINT_RECYCLED;
+	}
+	else if (relation->hints & UG_HINT_ACTIVE)
+		dlwidget = &cgtk->active;
+	else
+		dlwidget = &cgtk->queuing;
+
+	// Don't move task to the same UgDownloadWidget.
+	if (relation->user.data == &cgtk->active  &&  dlwidget == &cgtk->queuing) {
+		relation->user.data = dlwidget;
+		gtk_list_store_move_after (relation->user.storage,
+				relation->user.position, NULL);
+	}
+	else if (relation->user.data != dlwidget) {
+		relation->user.data = dlwidget;
+		if (dlwidget == &cgtk->recycled || dlwidget == &cgtk->finished) {
+			gtk_list_store_move_after (relation->user.storage,
+					relation->user.position, NULL);
+		}
+		else {
+			gtk_list_store_move_before (relation->user.storage,
+					relation->user.position, NULL);
+		}
+	}
+
+	model = GTK_TREE_MODEL (relation->user.storage);
+	path = gtk_tree_model_get_path (model, relation->user.position);
+	gtk_tree_model_row_changed (model, path, relation->user.position);
+	gtk_tree_path_free (path);
+}
+
+void	ug_category_gtk_remove (UgCategory* category, UgDataset* dataset)
+{
+	UgCategoryGtk* 	cgtk;
+	UgRelation*		relation;
+
+	cgtk = category->user.category;
+	relation = UG_DATASET_RELATION (dataset);
+	if (relation->user.storage) {
+		gtk_list_store_remove (
+				relation->user.storage,
+				relation->user.position);
+	}
+	relation->user.storage = NULL;
+	relation->category = NULL;
+	// delete data and files
+	ug_download_delete_temp (dataset);
+	ug_dataset_unref (dataset);
+}
+
+void	ug_category_gtk_clear (UgCategory* category, UgCategoryHints hint, guint from_nth)
+{
+	UgCategoryGtk*	cgtk;
+	UgDataset*		dataset;
+	GtkTreeModel*	model;
+	GtkTreeIter		iter;
+	GList*			list;
+	GList*			link;
+	gboolean		valid;
+
+	cgtk = category->user.category;
+	if (hint & UG_HINT_RECYCLED)
+		model = cgtk->recycled.model;
+	else if (hint & UG_HINT_FINISHED)
+		model = cgtk->finished.model;
+	else
+		model = cgtk->queuing.model;
+
+	// get tasks to clear
+	list = NULL;
+	valid = gtk_tree_model_iter_nth_child (model, &iter, NULL, from_nth);
+	while (valid) {
+		gtk_tree_model_get (model, &iter, 0, &dataset, -1);
+		list = g_list_prepend (list, dataset);
+		valid = gtk_tree_model_iter_next (model, &iter);
+	}
+	// remove tasks
+	for (link = list;  link;  link = link->next)
+		ug_category_gtk_remove (category, link->data);
+	g_list_free (list);
+}
+
+void	ug_category_gtk_move_to (UgCategory* category, UgDataset* dataset, UgCategory* category_dest)
+{
+	UgRelation*		relation;
+
+	relation = UG_DATASET_RELATION (dataset);
+	if (relation->category == category_dest)
+		return;
+
+	// move
+	relation->category  = category_dest;
+	relation->user.data = NULL;
+	ug_category_gtk_changed (category_dest, dataset);
+}
+
+gboolean	ug_category_gtk_move_selected_up (UgCategory* category, UgDownloadWidget* dwidget)
 {
 	UgDataset*		dataset;
-	UgRelationGtk*	relation;
-	UgRelationGtk*	relation_prev;
+	UgRelation*		relation;
+	UgRelation*		relation_prev;
 	GtkTreeIter		iter;
 	GtkTreePath*	path;
 	GList*			list;
 	GList*			link;
 	gint			index;
 	gint			index_prev;
-	gboolean		result = FALSE;
+	gboolean		retval = FALSE;
 
 	index_prev = -1;
 	list = ug_download_widget_get_selected_indices (dwidget);
@@ -364,33 +478,33 @@ gboolean	ug_category_gtk_move_selected_up (UgCategoryGtk* category, UgDownloadWi
 		// get previous
 		gtk_tree_model_iter_nth_child (dwidget->model, &iter, NULL, index_prev);
 		gtk_tree_model_get (dwidget->model, &iter, 0, &dataset, -1);
-		relation_prev = UG_DATASET_RELATION_GTK (dataset);
+		relation_prev = UG_DATASET_RELATION (dataset);
 		// get current
 		gtk_tree_model_iter_next (dwidget->model, &iter);
 		gtk_tree_model_get (dwidget->model, &iter, 0, &dataset, -1);
-		relation = UG_DATASET_RELATION_GTK (dataset);
+		relation = UG_DATASET_RELATION (dataset);
 		// swap
-		gtk_list_store_swap (relation->primary->store,
-				&relation->iter, &relation_prev->iter);
-		result = TRUE;
+		gtk_list_store_swap (relation->user.storage,
+				relation->user.position, relation_prev->user.position);
+		retval = TRUE;
 	}
 
 	g_list_free (list);
-	return result;
+	return retval;
 }
 
-gboolean	ug_category_gtk_move_selected_down (UgCategoryGtk* category, UgDownloadWidget* dwidget)
+gboolean	ug_category_gtk_move_selected_down (UgCategory* category, UgDownloadWidget* dwidget)
 {
 	UgDataset*		dataset;
-	UgRelationGtk*	relation;
-	UgRelationGtk*	relation_next;
+	UgRelation*		relation;
+	UgRelation*		relation_next;
 	GtkTreeIter		iter;
 	GtkTreePath*	path;
 	GList*			list;
 	GList*			link;
 	gint			index;
 	gint			index_next;
-	gboolean		result = FALSE;
+	gboolean		retval = FALSE;
 
 	index_next = gtk_tree_model_iter_n_children (dwidget->model, NULL);
 	list = ug_download_widget_get_selected_indices (dwidget);
@@ -415,33 +529,33 @@ gboolean	ug_category_gtk_move_selected_down (UgCategoryGtk* category, UgDownload
 		// get current
 		gtk_tree_model_iter_nth_child (dwidget->model, &iter, NULL, index);
 		gtk_tree_model_get (dwidget->model, &iter, 0, &dataset, -1);
-		relation = UG_DATASET_RELATION_GTK (dataset);
+		relation = UG_DATASET_RELATION (dataset);
 		// get next
 		gtk_tree_model_iter_next (dwidget->model, &iter);
 		gtk_tree_model_get (dwidget->model, &iter, 0, &dataset, -1);
-		relation_next = UG_DATASET_RELATION_GTK (dataset);
+		relation_next = UG_DATASET_RELATION (dataset);
 		// swap
-		gtk_list_store_swap (relation->primary->store,
-				&relation->iter, &relation_next->iter);
-		result = TRUE;
+		gtk_list_store_swap (relation->user.storage,
+				relation->user.position, relation_next->user.position);
+		retval = TRUE;
 	}
 
 	g_list_free (list);
-	return result;
+	return retval;
 }
 
-gboolean	ug_category_gtk_move_selected_to_top (UgCategoryGtk* category, UgDownloadWidget* dwidget)
+gboolean	ug_category_gtk_move_selected_to_top (UgCategory* category, UgDownloadWidget* dwidget)
 {
 	UgDataset*		dataset;
-	UgRelationGtk*	relation;
-	UgRelationGtk*	relation_top;
+	UgRelation*		relation;
+	UgRelation*		relation_top;
 	GtkTreeIter		iter;
 	GList*			list;
 	GList*			link;
 	gint			index;
 	gint			index_top;
 
-	// get movable jobs
+	// get movable tasks
 	relation_top = NULL;
 	index_top = 0;
 	list = ug_download_widget_get_selected_indices (dwidget);
@@ -454,12 +568,12 @@ gboolean	ug_category_gtk_move_selected_to_top (UgCategoryGtk* category, UgDownlo
 		else {
 			gtk_tree_model_iter_nth_child (dwidget->model, &iter, NULL, index);
 			gtk_tree_model_get (dwidget->model, &iter, 0, &dataset, -1);
-			link->data = UG_DATASET_RELATION_GTK (dataset);
+			link->data = UG_DATASET_RELATION (dataset);
 		}
 		if (relation_top == NULL) {
 			gtk_tree_model_iter_nth_child (dwidget->model, &iter, NULL, index_top);
 			gtk_tree_model_get (dwidget->model, &iter, 0, &dataset, -1);
-			relation_top = UG_DATASET_RELATION_GTK (dataset);
+			relation_top = UG_DATASET_RELATION (dataset);
 		}
 	}
 	list = g_list_remove_all (list, NULL);
@@ -469,8 +583,8 @@ gboolean	ug_category_gtk_move_selected_to_top (UgCategoryGtk* category, UgDownlo
 	// move to top
 	for (link = list;  link;  link = link->next) {
 		relation = link->data;
-		gtk_list_store_move_before (relation->primary->store,
-				&relation->iter, &relation_top->iter);
+		gtk_list_store_move_before (relation->user.storage,
+				relation->user.position, relation_top->user.position);
 	}
 	g_list_free (list);
 
@@ -479,11 +593,11 @@ gboolean	ug_category_gtk_move_selected_to_top (UgCategoryGtk* category, UgDownlo
 	return TRUE;
 }
 
-gboolean	ug_category_gtk_move_selected_to_bottom (UgCategoryGtk* category, UgDownloadWidget* dwidget)
+gboolean	ug_category_gtk_move_selected_to_bottom (UgCategory* category, UgDownloadWidget* dwidget)
 {
 	UgDataset*		dataset;
-	UgRelationGtk*	relation;
-	UgRelationGtk*	relation_bottom;
+	UgRelation*		relation;
+	UgRelation*		relation_bottom;
 	GtkTreeIter		iter;
 	GtkTreePath*	path;
 	GList*			list;
@@ -491,7 +605,7 @@ gboolean	ug_category_gtk_move_selected_to_bottom (UgCategoryGtk* category, UgDow
 	gint			index;
 	gint			index_bottom;
 
-	// get movable jobs
+	// get movable tasks
 	relation_bottom = NULL;
 	index_bottom = gtk_tree_model_iter_n_children (dwidget->model, NULL) - 1;
 	list = ug_download_widget_get_selected_indices (dwidget);
@@ -505,12 +619,12 @@ gboolean	ug_category_gtk_move_selected_to_bottom (UgCategoryGtk* category, UgDow
 		else {
 			gtk_tree_model_iter_nth_child (dwidget->model, &iter, NULL, index);
 			gtk_tree_model_get (dwidget->model, &iter, 0, &dataset, -1);
-			link->data = UG_DATASET_RELATION_GTK (dataset);
+			link->data = UG_DATASET_RELATION (dataset);
 		}
 		if (relation_bottom == NULL) {
 			gtk_tree_model_iter_nth_child (dwidget->model, &iter, NULL, index_bottom);
 			gtk_tree_model_get (dwidget->model, &iter, 0, &dataset, -1);
-			relation_bottom = UG_DATASET_RELATION_GTK (dataset);
+			relation_bottom = UG_DATASET_RELATION (dataset);
 		}
 	}
 	list = g_list_remove_all (list, NULL);
@@ -520,8 +634,8 @@ gboolean	ug_category_gtk_move_selected_to_bottom (UgCategoryGtk* category, UgDow
 	// move to bottom
 	for (link = list;  link;  link = link->next) {
 		relation = link->data;
-		gtk_list_store_move_after (relation->primary->store,
-				&relation->iter, &relation_bottom->iter);
+		gtk_list_store_move_after (relation->user.storage,
+				relation->user.position, relation_bottom->user.position);
 	}
 	g_list_free (list);
 
@@ -533,168 +647,36 @@ gboolean	ug_category_gtk_move_selected_to_bottom (UgCategoryGtk* category, UgDow
 	return TRUE;
 }
 
-gboolean	ug_category_gtk_start (UgCategoryGtk* category, UgDataset* dataset)
+// If no data cleared, return FALSE.
+gboolean	ug_category_gtk_clear_excess (UgCategory* category)
 {
-	if (ug_category_start ((UgCategory*) category, dataset)) {
-		if (g_queue_find (&category->active, dataset) == NULL)
-			g_queue_push_tail (&category->active, dataset);
-		return TRUE;
-	}
-	return FALSE;
-}
-
-void	ug_category_gtk_stop (UgCategoryGtk* category, UgDataset* dataset)
-{
-	ug_category_stop ((UgCategory*) category, dataset);
-	g_queue_remove (&category->active, dataset);
-}
-
-// returned value
-// FALSE: status no changed.
-// TRUE:  status changed. (start, stop, move, remove)
-gboolean	ug_category_gtk_refresh (UgCategoryGtk* cgtk, GList** completed)
-{
-	UgDataset*		dataset;
-	UgRelationGtk*	relation;
-	GList*			link;
-	gboolean		result = FALSE;
-
-	for (link = cgtk->active.head;  link;  link = link->next) {
-		dataset = link->data;
-		// refresh status & data
-		if (ug_category_refresh ((UgCategory*) cgtk, dataset))
-			continue;
-		// stop & clear
-		ug_category_stop ((UgCategory*) cgtk, dataset);
-		link->data = NULL;
-		// relation
-		relation = UG_DATASET_RELATION_GTK (dataset);
-		result = TRUE;
-		if (relation->hints & UG_HINT_FINISHED) {
-			ug_category_gtk_switch (cgtk, dataset, UG_HINT_FINISHED);
-			if (completed)
-				*completed = g_list_prepend (*completed, dataset);
-		}
-	}
-	g_queue_remove_all (&cgtk->active, NULL);
-
-	return result;
-}
-
-gboolean	ug_category_gtk_activate (UgCategoryGtk* cgtk)
-{
-	UgDataset*		dataset;
-	UgRelationGtk*	relation;
-	GtkTreeIter		iter;
-	gboolean		valid;
-	gboolean		result = FALSE;
-
-	valid = gtk_tree_model_get_iter_first (cgtk->queuing.model, &iter);
-	while (valid  &&  cgtk->active.length < cgtk->limit.active) {
-		gtk_tree_model_get (cgtk->queuing.model, &iter, 0, &dataset, -1);
-		valid = gtk_tree_model_iter_next (cgtk->queuing.model, &iter);
-		// check & start
-		relation = UG_DATASET_RELATION_GTK (dataset);
-		if (relation->hints & (UG_HINT_UNRUNNABLE | UG_HINT_ACTIVE))
-			continue;
-		if (ug_category_gtk_start (cgtk, dataset) == FALSE) {
-			g_free (relation->message);
-			relation->message_type = UG_MESSAGE_ERROR;
-			relation->message = g_strdup (_("Unsupported scheme (protocol)."));
-		}
-		result = TRUE;
-	}
-	return result;
-}
-
-gboolean	ug_category_gtk_clear_excess (UgCategoryGtk* cgtk)
-{
-	guint		n;
-	gboolean	result = FALSE;
+	UgCategoryGtk*	cgtk;
+	guint			n;
+	gboolean		retval = FALSE;
 
+	cgtk = category->user.category;
 	n = gtk_tree_model_iter_n_children (cgtk->finished.model, NULL);
-	if (n > cgtk->limit.finished)
-		result = TRUE;
+	if (n > category->finished_limit)
+		retval = TRUE;
 	n = gtk_tree_model_iter_n_children (cgtk->recycled.model, NULL);
-	if (n > cgtk->limit.recycled)
-		result = TRUE;
+	if (n > category->recycled_limit)
+		retval = TRUE;
 
-	ug_category_gtk_clear (cgtk, UG_HINT_FINISHED, cgtk->limit.finished);
-	ug_category_gtk_clear (cgtk, UG_HINT_RECYCLED, cgtk->limit.recycled);
+	ug_category_gtk_clear (category, UG_HINT_FINISHED, category->finished_limit);
+	ug_category_gtk_clear (category, UG_HINT_RECYCLED, category->recycled_limit);
 
-	return result;
+	return retval;
 }
 
-// ----------------------------------------------
-// static function
-static gboolean ug_category_gtk_watch (UgCategoryGtk* ccmd, UgDataset* dataset, UgMessage* message, gpointer user_data)
+// ----------------------------------------------------------------------------
+// free GtkTreeIter in UgRelation::user.position
+static void		ug_slice_free_tree_iter (GtkTreeIter* iter)
 {
-	UgRelationGtk*	relation;
-
-	relation = UG_DATASET_RELATION_GTK (dataset);
-
-	switch (message->type) {
-	case UG_MESSAGE_ERROR:
-	case UG_MESSAGE_WARNING:
-		g_free (relation->message);
-		relation->message = g_strdup (message->string);
-		relation->message_type = message->type;
-		break;
-
-	default:
-		break;
-	}
-	return FALSE;
-}
-
-// Function used by GtkTreeModelSort.
-static gint	ug_download_model_compare (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer user_data)
-{
-	UgDataset*		dataset;
-	UgDataCommon*	common;
-	gchar*			name1;
-	gchar*			name2;
-
-	gtk_tree_model_get (model, a, 0, &dataset, -1);
-	if (dataset == NULL)
-		return 0;
-	common = UG_DATASET_COMMON (dataset);
-	if (common->name)
-		name1 = common->name;
-	else if (common->file)
-		name1 = common->file;
-	else
-		name1 = "unnamed";
-
-	gtk_tree_model_get (model, b, 0, &dataset, -1);
-	if (dataset == NULL)
-		return 0;
-	common = UG_DATASET_COMMON (dataset);
-	if (common->name)
-		name2 = common->name;
-	else if (common->file)
-		name2 = common->file;
-	else
-		name2 = "unnamed";
-	return strcmp (name1, name2);
+	g_slice_free1 (sizeof (GtkTreeIter), iter);
 }
 
 // Function used by GtkTreeModelFilter.
-static gboolean	ug_download_model_filter_category (GtkTreeModel *model, GtkTreeIter *iter, UgCategoryGtk* category)
-{
-	UgDataset*		dataset;
-	UgRelationGtk*	relation;
-
-	gtk_tree_model_get (model, iter, 0, &dataset, -1);
-	if (dataset) {
-		relation = UG_DATASET_RELATION_GTK (dataset);
-		if (relation->category == category)
-			return TRUE;
-	}
-	return FALSE;
-}
-
-static gboolean	ug_download_model_filter_queuing  (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
+static gboolean	ug_download_model_filter_category (GtkTreeModel *model, GtkTreeIter *iter, UgCategory* category)
 {
 	UgDataset*		dataset;
 	UgRelation*		relation;
@@ -702,7 +684,35 @@ static gboolean	ug_download_model_filter_queuing  (GtkTreeModel *model, GtkTreeI
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
 	if (dataset) {
 		relation = UG_DATASET_RELATION (dataset);
-		if (relation->hints & (UG_HINT_FINISHED | UG_HINT_RECYCLED))
+		if (relation->category == category)
+			return TRUE;
+	}
+	return FALSE;
+}
+
+static gboolean	ug_download_model_filter_active (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
+{
+	UgDataset*		dataset;
+	UgRelation*		relation;
+
+	gtk_tree_model_get (model, iter, 0, &dataset, -1);
+	if (dataset) {
+		relation = UG_DATASET_RELATION (dataset);
+		if (relation->hints & UG_HINT_ACTIVE)
+			return TRUE;
+	}
+	return FALSE;
+}
+
+static gboolean	ug_download_model_filter_queuing (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
+{
+	UgDataset*		dataset;
+	UgRelation*		relation;
+
+	gtk_tree_model_get (model, iter, 0, &dataset, -1);
+	if (dataset) {
+		relation = UG_DATASET_RELATION (dataset);
+		if (relation->hints & (UG_HINT_FINISHED | UG_HINT_RECYCLED | UG_HINT_ACTIVE))
 			return FALSE;
 	}
 	return TRUE;
@@ -738,39 +748,3 @@ static gboolean	ug_download_model_filter_recycled (GtkTreeModel *model, GtkTreeI
 	return FALSE;
 }
 
-
-// ----------------------------------------------------------------------------
-// UgRelationGtk
-//
-static void	ug_relation_gtk_assign (UgRelation* relation, UgRelation* src);
-
-static UgDataEntry	relation_gtk_data_entry[] =
-{
-	UG_RELATION_DATA_ENTRY,
-	{NULL},			// null-terminated
-};
-
-static UgDataClass	relation_gtk_data_class =
-{
-	UG_RELATION_CLASS_NAME,		// name
-	NULL,						// reserve
-	sizeof (UgRelationGtk),		// instance_size
-	relation_gtk_data_entry,	// entry
-
-	(UgInitFunc)     NULL,
-	(UgFinalizeFunc) NULL,
-	(UgAssignFunc)   ug_relation_gtk_assign,
-};
-// extern
-const	UgDataClass*	UgRelationGtkClass = &relation_gtk_data_class;
-
-static void	ug_relation_gtk_assign (UgRelation* relation, UgRelation* src)
-{
-	if (src->hints & UG_HINT_PAUSED) {
-		relation->hints |=  UG_HINT_PAUSED;
-		relation->hints &= ~UG_HINT_ACTIVE;
-	}
-	else
-		relation->hints &= ~UG_HINT_PAUSED;
-}
-
