@@ -1,7 +1,7 @@
 /*
  *
- *   Copyright (C) 2005-2010 by Raymond Huang
- *   plushuang at users.sourceforge.net
+ *   Copyright (C) 2005-2011 by C.H. Huang
+ *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -40,13 +40,15 @@
 
 
 #define	UG_CATEGORY_GTK_ALL_NAME			_("All")
+#define	UG_CATEGORY_GTK_ACTIVE_NAME			_("Active")
 #define	UG_CATEGORY_GTK_QUEUING_NAME		_("Queuing")
 #define	UG_CATEGORY_GTK_FINISHED_NAME		_("Finished")
 #define	UG_CATEGORY_GTK_RECYCLED_NAME		_("Recycled")
 
 // This is used by GtkTreeStore
-enum UgCategoryStoreColumn {
-	UG_CATEGORY_STORE_INSTANCE,		// UgCategoryGtk
+enum UgCategoryStoreColumn
+{
+	UG_CATEGORY_STORE_INSTANCE,		// UgCategory
 	UG_CATEGORY_STORE_DOWNLOAD,		// UgDownloadWidget
 	UG_CATEGORY_N_COLUMN
 };
@@ -67,42 +69,48 @@ void	ug_category_widget_init (UgCategoryWidget* cwidget)
 	GtkBox*				vbox;
 
 	// primary category
-	cwidget->primary = ug_category_gtk_new (NULL);
-	cwidget->primary->name = g_strdup (UG_CATEGORY_GTK_ALL_NAME);
+	cwidget->primary.category = ug_category_new_with_gtk (NULL);
+	cwidget->primary.category->name = g_strdup (UG_CATEGORY_GTK_ALL_NAME);
+	cwidget->primary.cgtk = cwidget->primary.category->user.category;
 	// primary view & store
-	cwidget->primary_view = ug_category_view_new ();
+	cwidget->primary.view = ug_category_view_new ();
 	primary_store = gtk_list_store_new (UG_CATEGORY_N_COLUMN, G_TYPE_POINTER, G_TYPE_POINTER);
-	gtk_tree_view_set_model (cwidget->primary_view, GTK_TREE_MODEL (primary_store));
+	gtk_tree_view_set_model (cwidget->primary.view, GTK_TREE_MODEL (primary_store));
 	g_object_unref (primary_store);
 
 	gtk_list_store_append (primary_store, &iter);
 	gtk_list_store_set (primary_store, &iter,
-			UG_CATEGORY_STORE_INSTANCE, cwidget->primary,
-			UG_CATEGORY_STORE_DOWNLOAD, &cwidget->primary->all,
+			UG_CATEGORY_STORE_INSTANCE, cwidget->primary.category,
+			UG_CATEGORY_STORE_DOWNLOAD, &cwidget->primary.cgtk->all,
 			-1);
 	gtk_list_store_append (primary_store, &iter);
 	gtk_list_store_set (primary_store, &iter,
-			UG_CATEGORY_STORE_INSTANCE, cwidget->primary,
-			UG_CATEGORY_STORE_DOWNLOAD, &cwidget->primary->queuing,
+			UG_CATEGORY_STORE_INSTANCE, cwidget->primary.category,
+			UG_CATEGORY_STORE_DOWNLOAD, &cwidget->primary.cgtk->active,
 			-1);
 	gtk_list_store_append (primary_store, &iter);
 	gtk_list_store_set (primary_store, &iter,
-			UG_CATEGORY_STORE_INSTANCE, cwidget->primary,
-			UG_CATEGORY_STORE_DOWNLOAD, &cwidget->primary->finished,
+			UG_CATEGORY_STORE_INSTANCE, cwidget->primary.category,
+			UG_CATEGORY_STORE_DOWNLOAD, &cwidget->primary.cgtk->queuing,
 			-1);
 	gtk_list_store_append (primary_store, &iter);
 	gtk_list_store_set (primary_store, &iter,
-			UG_CATEGORY_STORE_INSTANCE, cwidget->primary,
-			UG_CATEGORY_STORE_DOWNLOAD, &cwidget->primary->recycled,
+			UG_CATEGORY_STORE_INSTANCE, cwidget->primary.category,
+			UG_CATEGORY_STORE_DOWNLOAD, &cwidget->primary.cgtk->finished,
+			-1);
+	gtk_list_store_append (primary_store, &iter);
+	gtk_list_store_set (primary_store, &iter,
+			UG_CATEGORY_STORE_INSTANCE, cwidget->primary.category,
+			UG_CATEGORY_STORE_DOWNLOAD, &cwidget->primary.cgtk->recycled,
 			-1);
 
 	// secondary categories: model and view
 	cwidget->store = gtk_tree_store_new (UG_CATEGORY_N_COLUMN, G_TYPE_POINTER, G_TYPE_POINTER);
 	cwidget->view = ug_category_view_new ();
 	gtk_tree_view_set_model (cwidget->view, GTK_TREE_MODEL (cwidget->store));
-	gtk_widget_set_size_request (GTK_WIDGET (cwidget->view), 165, 100);
 	// secondary categories: scrolled for view
 	cwidget->scroll = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_set_size_request (cwidget->scroll, 165, 100);
 	scroll = GTK_SCROLLED_WINDOW (cwidget->scroll);
 	gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy (scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -117,57 +125,64 @@ void	ug_category_widget_init (UgCategoryWidget* cwidget)
 	// container widget
 	cwidget->self = gtk_vbox_new (FALSE, 2);
 	vbox = GTK_BOX (cwidget->self);
-	gtk_box_pack_start (vbox, GTK_WIDGET (cwidget->primary_view), FALSE, FALSE, 0);
+	gtk_box_pack_start (vbox, GTK_WIDGET (cwidget->primary.view), FALSE, FALSE, 0);
 	gtk_box_pack_start (vbox, gtk_label_new (_("Category")), FALSE, FALSE, 0);
 	gtk_box_pack_start (vbox, GTK_WIDGET (cwidget->scroll), TRUE, TRUE, 0);
 	gtk_widget_show_all (cwidget->self);
 
 	// signal
-	g_signal_connect (cwidget->primary_view, "cursor-changed",
+	g_signal_connect (cwidget->primary.view, "cursor-changed",
 			G_CALLBACK (ug_category_cursor_changed), cwidget);
 	g_signal_connect (cwidget->view, "cursor-changed",
 			G_CALLBACK (ug_category_cursor_changed), cwidget);
 }
 
-void	ug_category_widget_append (UgCategoryWidget* cwidget, UgCategoryGtk* category)
+void	ug_category_widget_append (UgCategoryWidget* cwidget, UgCategory* category)
 {
+	UgCategoryGtk*	cgtk;
 	GtkTreeIter		iter;
 
-	if (cwidget->primary == category)
+	if (cwidget->primary.category == category)
 		return;
 
+	cgtk = category->user.category;
 	// append data to GtkTreeStore
-	gtk_tree_store_append (cwidget->store, &category->tree_iter, NULL);
-	gtk_tree_store_set (cwidget->store, &category->tree_iter,
+	gtk_tree_store_append (cwidget->store, &cgtk->tree_iter, NULL);
+	gtk_tree_store_set (cwidget->store, &cgtk->tree_iter,
 			UG_CATEGORY_STORE_INSTANCE, category,
-			UG_CATEGORY_STORE_DOWNLOAD, &category->all,
+			UG_CATEGORY_STORE_DOWNLOAD, &cgtk->all,
 			-1);
 
-	gtk_tree_store_append (cwidget->store, &iter, &category->tree_iter);
+	gtk_tree_store_append (cwidget->store, &iter, &cgtk->tree_iter);
 	gtk_tree_store_set (cwidget->store, &iter,
 			UG_CATEGORY_STORE_INSTANCE, category,
-			UG_CATEGORY_STORE_DOWNLOAD, &category->queuing,
+			UG_CATEGORY_STORE_DOWNLOAD, &cgtk->active,
 			-1);
-	gtk_tree_store_append (cwidget->store, &iter, &category->tree_iter);
+	gtk_tree_store_append (cwidget->store, &iter, &cgtk->tree_iter);
 	gtk_tree_store_set (cwidget->store, &iter,
 			UG_CATEGORY_STORE_INSTANCE, category,
-			UG_CATEGORY_STORE_DOWNLOAD, &category->finished,
+			UG_CATEGORY_STORE_DOWNLOAD, &cgtk->queuing,
 			-1);
-	gtk_tree_store_append (cwidget->store, &iter, &category->tree_iter);
+	gtk_tree_store_append (cwidget->store, &iter, &cgtk->tree_iter);
 	gtk_tree_store_set (cwidget->store, &iter,
 			UG_CATEGORY_STORE_INSTANCE, category,
-			UG_CATEGORY_STORE_DOWNLOAD, &category->recycled,
+			UG_CATEGORY_STORE_DOWNLOAD, &cgtk->finished,
+			-1);
+	gtk_tree_store_append (cwidget->store, &iter, &cgtk->tree_iter);
+	gtk_tree_store_set (cwidget->store, &iter,
+			UG_CATEGORY_STORE_INSTANCE, category,
+			UG_CATEGORY_STORE_DOWNLOAD, &cgtk->recycled,
 			-1);
 }
 
-void	ug_category_widget_remove (UgCategoryWidget* cwidget, UgCategoryGtk* category)
+void	ug_category_widget_remove (UgCategoryWidget* cwidget, UgCategory* category)
 {
 	GtkTreeSelection*	selection;
 	GtkTreeModel*		model;
 	GtkTreePath*		path;
 	gint				n_rows;
 
-	if (cwidget->primary == category)
+	if (cwidget->primary.category == category)
 		return;
 
 	model  = gtk_tree_view_get_model (cwidget->view);
@@ -176,11 +191,11 @@ void	ug_category_widget_remove (UgCategoryWidget* cwidget, UgCategoryGtk* catego
 		selection = gtk_tree_view_get_selection (cwidget->view);
 		gtk_tree_selection_unselect_all (selection);
 		// switch to primary_view
-		gtk_tree_view_get_cursor (cwidget->primary_view, &path, NULL);
-		gtk_tree_view_set_cursor (cwidget->primary_view, path,  NULL, FALSE);
+		gtk_tree_view_get_cursor (cwidget->primary.view, &path, NULL);
+		gtk_tree_view_set_cursor (cwidget->primary.view, path,  NULL, FALSE);
 		gtk_tree_path_free (path);
 	}
-	else if (cwidget->current == category) {
+	else if (cwidget->current.category == category) {
 		gtk_tree_view_get_cursor (cwidget->view, &path, NULL);
 		if (gtk_tree_path_get_depth (path) == 2)
 			gtk_tree_path_up (path);
@@ -192,8 +207,9 @@ void	ug_category_widget_remove (UgCategoryWidget* cwidget, UgCategoryGtk* catego
 		gtk_tree_path_free (path);
 	}
 
-	gtk_tree_store_remove (cwidget->store, &category->tree_iter);
-	ug_category_gtk_free (category);
+	gtk_tree_store_remove (cwidget->store,
+			&((UgCategoryGtk*)category->user.category)->tree_iter);
+	ug_category_free (category);
 }
 
 void	ug_category_widget_add_list (UgCategoryWidget* cwidget, GList*	list)
@@ -234,7 +250,7 @@ void	ug_category_widget_set_cursor (UgCategoryWidget* cwidget, gint nth_category
 
 	if (nth_category == -1) {
 		path = gtk_tree_path_new_from_indices (nth_dwidget, -1);
-		gtk_tree_view_set_cursor (cwidget->primary_view, path, NULL, FALSE);
+		gtk_tree_view_set_cursor (cwidget->primary.view, path, NULL, FALSE);
 	}
 	else {
 		path = gtk_tree_path_new_from_indices (nth_category, nth_dwidget-1 , -1);
@@ -251,12 +267,18 @@ static void	ug_category_cursor_changed (GtkTreeView* view, UgCategoryWidget* cwi
 	GtkTreePath*		path;
 	GtkTreeIter			iter;
 
-	if (cwidget->current_view != view) {
-		cwidget->current_view = view;
-		if (cwidget->primary_view == view)
+	// get setting of download column
+	if (cwidget->current.widget == &cwidget->current.cgtk->all) {
+		cwidget->sort.nth   = cwidget->current.widget->sort.nth;
+		cwidget->sort.order = cwidget->current.widget->sort.order;
+	}
+
+	if (cwidget->current.view != view) {
+		cwidget->current.view = view;
+		if (cwidget->primary.view == view)
 			selection = gtk_tree_view_get_selection (cwidget->view);
 		else
-			selection = gtk_tree_view_get_selection (cwidget->primary_view);
+			selection = gtk_tree_view_get_selection (cwidget->primary.view);
 		gtk_tree_selection_unselect_all (selection);
 	}
 
@@ -266,27 +288,39 @@ static void	ug_category_cursor_changed (GtkTreeView* view, UgCategoryWidget* cwi
 		gtk_tree_model_get_iter (model, &iter, path);
 		gtk_tree_path_free (path);
 		gtk_tree_model_get (model, &iter,
-				UG_CATEGORY_STORE_INSTANCE, &cwidget->current,
-				UG_CATEGORY_STORE_DOWNLOAD, &cwidget->current_widget,
+				UG_CATEGORY_STORE_INSTANCE, &cwidget->current.category,
+				UG_CATEGORY_STORE_DOWNLOAD, &cwidget->current.widget,
 				-1);
 	}
 	else {
-		cwidget->current = cwidget->primary;
-		cwidget->current_widget = &cwidget->primary->all;
+		cwidget->current.category = cwidget->primary.category;
+		cwidget->current.widget = &cwidget->primary.cgtk->all;
+	}
+
+	// set UgCategoryGtk from UgCategory
+	cwidget->current.cgtk = cwidget->current.category->user.category;
+
+	// set setting of download column
+	if (cwidget->current.widget == &cwidget->current.cgtk->all) {
+		ug_download_view_set_sort_order (cwidget->current.widget->view,
+				cwidget->sort.nth, cwidget->sort.order);
+		cwidget->current.widget->sort.nth   = cwidget->sort.nth;
+		cwidget->current.widget->sort.order = cwidget->sort.order;
 	}
 }
 
 // Function used by GtkTreeModelFilter.
 static gboolean	ug_category_model_filter (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
-	UgCategoryGtk*		category;
+	UgCategory*			category;
 	UgDownloadWidget*	dwidget;
 
 	gtk_tree_model_get (model, iter,
 			UG_CATEGORY_STORE_INSTANCE, &category,
 			UG_CATEGORY_STORE_DOWNLOAD, &dwidget,
 			-1);
-	if (&category->all == dwidget)
+
+	if (category && &((UgCategoryGtk*)category->user.category)->all == dwidget)
 		return TRUE;
 	return FALSE;
 }
@@ -307,7 +341,8 @@ static void category_col_set_icon (GtkTreeViewColumn *tree_column,
                                    GtkTreeIter       *iter,
                                    gpointer           data)
 {
-	UgCategoryGtk*		category;
+	UgCategory*			category;
+	UgCategoryGtk*		cgtk;
 	UgDownloadWidget*	dwidget;
 	const gchar*		stock_id;
 
@@ -315,14 +350,15 @@ static void category_col_set_icon (GtkTreeViewColumn *tree_column,
 			UG_CATEGORY_STORE_INSTANCE, &category,
 			UG_CATEGORY_STORE_DOWNLOAD, &dwidget,
 			-1);
+	cgtk = category->user.category;
 
-	if (dwidget == &category->all)
+	if (dwidget == &cgtk->all)
 		stock_id = UG_CATEGORY_ICON_ALL;
-	else if (dwidget == &category->finished)
+	else if (dwidget == &cgtk->finished)
 		stock_id = UG_CATEGORY_ICON_FINISHED;
-	else if (dwidget == &category->recycled)
+	else if (dwidget == &cgtk->recycled)
 		stock_id = UG_CATEGORY_ICON_RECYCLED;
-	else if (category->active.length == 0)	// dwidget == &category->queuing
+	else if (dwidget == &cgtk->queuing)
 		stock_id = UG_CATEGORY_ICON_PAUSED;
 	else
 		stock_id = UG_CATEGORY_ICON_EXECUTE;
@@ -336,7 +372,8 @@ static void category_col_set_name (GtkTreeViewColumn *tree_column,
                                    GtkTreeIter       *iter,
                                    gpointer           data)
 {
-	UgCategoryGtk*		category;
+	UgCategory*			category;
+	UgCategoryGtk*		cgtk;
 	UgDownloadWidget*	dwidget;
 	gchar*				string;
 
@@ -344,12 +381,15 @@ static void category_col_set_name (GtkTreeViewColumn *tree_column,
 			UG_CATEGORY_STORE_INSTANCE, &category,
 			UG_CATEGORY_STORE_DOWNLOAD, &dwidget,
 			-1);
+	cgtk = category->user.category;
 
-	if (dwidget == &category->queuing)
+	if (dwidget == &cgtk->active)
+		string = UG_CATEGORY_GTK_ACTIVE_NAME;
+	else if (dwidget == &cgtk->queuing)
 		string = UG_CATEGORY_GTK_QUEUING_NAME;
-	else if (dwidget == &category->finished)
+	else if (dwidget == &cgtk->finished)
 		string = UG_CATEGORY_GTK_FINISHED_NAME;
-	else if (dwidget == &category->recycled)
+	else if (dwidget == &cgtk->recycled)
 		string = UG_CATEGORY_GTK_RECYCLED_NAME;
 	else
 		string = category->name;
@@ -406,7 +446,7 @@ GtkTreeView* ug_category_view_new (void)
 //	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_append_column (view, column);
 
-	// column jobs
+	// column Quantity = number of tasks
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_title (column, _("Quantity"));
 	renderer = gtk_cell_renderer_text_new ();
@@ -426,25 +466,28 @@ GtkTreeView* ug_category_view_new (void)
 	return view;
 }
 
-UgCategoryGtk*	ug_category_view_get_nth (GtkTreeView* view, gint n)
+UgCategory*	ug_category_view_get_nth (GtkTreeView* view, gint n)
 {
-	GtkTreeModel*		model;
-	GtkTreeIter			iter;
-	UgCategoryGtk*		category;
+	GtkTreeModel*	model;
+	GtkTreeIter		iter;
+	UgCategory*		category;
 
 	model = gtk_tree_view_get_model (view);
 	if (gtk_tree_model_iter_nth_child (model, &iter, NULL, n) == FALSE)
 		return NULL;
-	gtk_tree_model_get (model, &iter, UG_CATEGORY_STORE_INSTANCE, &category, -1);
+
+	gtk_tree_model_get (model, &iter,
+			UG_CATEGORY_STORE_INSTANCE, &category,
+			-1);
 	return category;
 }
 
-UgCategoryGtk*	ug_category_view_get_cursor (GtkTreeView* view)
+UgCategory*	ug_category_view_get_cursor (GtkTreeView* view)
 {
-	GtkTreeModel*		model;
-	GtkTreePath*		path;
-	GtkTreeIter			iter;
-	UgCategoryGtk*		category;
+	GtkTreeModel*	model;
+	GtkTreePath*	path;
+	GtkTreeIter		iter;
+	UgCategory*		category;
 
 	model = gtk_tree_view_get_model (view);
 	gtk_tree_view_get_cursor (view, &path, NULL);
@@ -454,7 +497,10 @@ UgCategoryGtk*	ug_category_view_get_cursor (GtkTreeView* view)
 	}
 	else if (gtk_tree_model_get_iter_first (model, &iter) == FALSE)
 		return NULL;
-	gtk_tree_model_get (model, &iter, UG_CATEGORY_STORE_INSTANCE, &category, -1);
+
+	gtk_tree_model_get (model, &iter,
+			UG_CATEGORY_STORE_INSTANCE, &category,
+			-1);
 	return category;
 }
 
