@@ -1,7 +1,7 @@
 /*
  *
- *   Copyright (C) 2005-2010 by Raymond Huang
- *   plushuang at users.sourceforge.net
+ *   Copyright (C) 2005-2011 by C.H. Huang
+ *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -44,193 +44,118 @@
 #include <UgData-download.h>
 #include <UgPlugin.h>
 
-const	UgDataClass*	UgCategoryClass = NULL;
-const	UgDataClass*	UgRelationClass = NULL;
 
 // ----------------------------------------------------------------------------
 // UgCategory
+
+static void	ug_category_assign   (UgCategory* category, UgCategory* src);
+// UgCategory.indices parse/write for UgMarkup
+static void	ug_int_list_in_markup (GList** list, GMarkupParseContext* context);
+static void	ug_int_list_to_markup (GList** list, UgMarkup* markup);
+
+static const UgDataEntry	ug_category_entry[] =
+{
+	{"name",			G_STRUCT_OFFSET (UgCategory, name),				UG_DATA_STRING,		NULL,	NULL},
+	{"ActiveLimit",		G_STRUCT_OFFSET (UgCategory, active_limit),		UG_DATA_UINT,		NULL,	NULL},
+	{"FinishedLimit",	G_STRUCT_OFFSET (UgCategory, finished_limit),	UG_DATA_UINT,		NULL,	NULL},
+	{"RecycledLimit",	G_STRUCT_OFFSET (UgCategory, recycled_limit),	UG_DATA_UINT,		NULL,	NULL},
+	{"DownloadDefault",	G_STRUCT_OFFSET (UgCategory, defaults),			UG_DATA_INSTANCE,	&ug_dataset_iface,	NULL},
+	{"DownloadIndices",	G_STRUCT_OFFSET (UgCategory, indices),			UG_DATA_CUSTOM,		ug_int_list_in_markup,	ug_int_list_to_markup},
+	{NULL},			// null-terminated
+};
+// extern
+const UgDataInterface	ug_category_iface =
+{
+	sizeof (UgCategory),	// instance_size
+	"category",				// name
+	ug_category_entry,		// entry
+
+	(UgInitFunc)     ug_category_init,
+	(UgFinalizeFunc) ug_category_finalize,
+	(UgAssignFunc)   ug_category_assign,
+};
+// extern
+const UgDataInterface*	ug_category_iface_pointer = &ug_category_iface;
+
+
 void	ug_category_init (UgCategory* category)
 {
+//	category->iface = ug_category_iface_pointer;
+
 	category->defaults = ug_dataset_new ();
-	category->limit.active   = 3;
-	category->limit.finished = 300;
-	category->limit.recycled = 300;
+	category->active_limit   = 3;
+	category->finished_limit = 300;
+	category->recycled_limit = 300;
 }
 
 void	ug_category_finalize (UgCategory* category)
 {
+	if (category->destroy.func)
+		category->destroy.func (category->destroy.data);
+
 	g_free (category->name);
 	if (category->defaults)
 		ug_dataset_unref (category->defaults);
 }
 
-void	ug_category_assign (UgCategory* category, UgCategory* src)
+static void	ug_category_assign (UgCategory* category, UgCategory* src)
 {
 	ug_str_set (&category->name, src->name, -1);
-	category->limit.active = src->limit.active;
-	category->limit.finished = src->limit.finished;
-	category->limit.recycled = src->limit.recycled;
+
+	category->active_limit   = src->active_limit;
+	category->finished_limit = src->finished_limit;
+	category->recycled_limit = src->recycled_limit;
+
 	ug_data_assign (category->defaults, src->defaults);
 }
 
-gboolean	ug_category_refresh (UgCategory* category, UgDataset* dataset)
+UgCategory*	ug_category_new (void)
 {
-	UgRelation*		relation;
-	UgMessage*		msg_list;
-	UgMessage*		msg;
-	UgState			plugin_state;
-	union {
-		UgDataCommon*	common;
-		UgDataHttp*		http;
-	} data;
-
-	// if not active, return FALSE.
-	relation = UG_DATASET_RELATION (dataset);
-	if (relation->plugin == NULL)
-		return FALSE;
-	// dispatch messages
-	msg_list = ug_plugin_pop_all (relation->plugin);
-	for (msg = msg_list;  msg;  msg = msg->next) {
-		// call watch functions
-		if (relation->watch.func) {
-			if (relation->watch.func (category, dataset, msg, relation->watch.data) == TRUE)
-				continue;
-		}
-		if (category->watch.func) {
-			if (category->watch.func (category, dataset, msg, category->watch.data) == TRUE)
-				continue;
-		}
-
-		// default message handler
-		switch (msg->type) {
-		case UG_MESSAGE_STATE:
-			if (msg->data.v_int != UG_STATE_ACTIVE)
-				relation->hints &= ~UG_HINT_ACTIVE;
-			break;
-
-		case UG_MESSAGE_PROGRESS:
-			ug_plugin_get (relation->plugin, UG_DATA_TYPE_INSTANCE,
-						ug_dataset_realloc (dataset, UgProgressClass, 0));
-			break;
-
-		case UG_MESSAGE_ERROR:
-			relation->hints |= UG_HINT_ERROR;
-			break;
-
-		case UG_MESSAGE_INFO:
-			switch (msg->code) {
-			case UG_MESSAGE_INFO_RETRY:
-				UG_DATASET_COMMON (dataset)->retry_count++;
-				break;
-
-			case UG_MESSAGE_INFO_COMPLETE:
-				relation->hints |= UG_HINT_COMPLETED;
-				break;
-
-			case UG_MESSAGE_INFO_FINISH:
-				relation->hints |= UG_HINT_FINISHED;
-				relation->hints &= ~UG_HINT_ERROR;
-				break;
-
-			case UG_MESSAGE_INFO_RESUMABLE:
-				relation->hints &= ~UG_HINT_RESUMED_FIELD;
-				relation->hints |= UG_HINT_RESUMABLE;
-				break;
-
-			case UG_MESSAGE_INFO_UNRESUMABLE:
-				relation->hints &= ~UG_HINT_RESUMED_FIELD;
-				relation->hints |= UG_HINT_UNRESUMABLE;
-				break;
-
-			default:
-				break;
-			}
-			// End of switch (msg->code)
-			break;
-		// End of UG_MESSAGE_INFO
-
-		case UG_MESSAGE_DATA:
-			switch (msg->code) {
-			case UG_MESSAGE_DATA_FILE_CHANGED:
-				if (msg->data.v_string) {
-					data.common = UG_DATASET_COMMON (dataset);
-					ug_str_set (&data.common->file, msg->data.v_string, -1);
-				}
-				break;
-
-			case UG_MESSAGE_DATA_URL_CHANGED:
-			// HTTP message
-			case UG_MESSAGE_DATA_HTTP_LOCATION:		// redirection
-				if (msg->data.v_string) {
-					data.common = UG_DATASET_COMMON (dataset);
-					ug_str_set (&data.common->url, msg->data.v_string, -1);
-					data.http = ug_dataset_realloc (dataset, UgDataHttpClass, 0);
-					data.http->redirection_count++;
-				}
-				break;
-
-			default:
-				break;
-			}
-			// End of switch (msg->code)
-			break;
-		// End of UG_MESSAGE_DATA
-
-		default:
-			break;
-		}
-		// End of switch (msg->type)
-	}
-	// free message list
-	ug_data_list_free (msg_list);
-
-	// if not active, return FALSE.
-	ug_plugin_get_state (relation->plugin, &plugin_state);
-	if (plugin_state != UG_STATE_ACTIVE)
-		return FALSE;
-	return TRUE;
+	return ug_data_new (ug_category_iface_pointer);
 }
 
-gboolean	ug_category_start (UgCategory* category, UgDataset* dataset)
+void	ug_category_free (UgCategory* category)
 {
-	UgRelation*		relation;
-
-	relation = UG_DATASET_RELATION (dataset);
-	// create plug-in
-	if (relation->plugin == NULL) {
-		relation->plugin = ug_plugin_new_by_data (dataset);
-		if (relation->plugin == NULL) {
-			relation->hints |=  UG_HINT_ERROR;
-			return FALSE;
-		}
-	}
-	// change plug-in state
-	ug_plugin_set_state (relation->plugin, UG_STATE_ACTIVE);
-	// set relation->hints
-	relation->hints &= ~UG_HINT_PAUSED;
-	relation->hints |=  UG_HINT_ACTIVE;
-	// reset
-//	UG_DATASET_COMMON (dataset)->retry_count = 0;
-//	http = ug_dataset_get (dataset, UgDataHttpClass, 0);
-//	http->redirection_count = 0;
-	return TRUE;
+	ug_data_free (category);
 }
 
-void	ug_category_stop (UgCategory* category, UgDataset* dataset)
+// add dataset to category and increase reference count of dataset.
+void	ug_category_add (UgCategory* category, UgDataset* dataset)
 {
-	UgRelation*		relation;
+	UgDataLog*			datalog;
 
-	relation = UG_DATASET_RELATION (dataset);
-	if (relation->plugin) {
-		ug_plugin_set_state (relation->plugin, UG_STATE_NULL);
-		ug_plugin_unref (relation->plugin);
-		relation->plugin = NULL;
-	}
-	// set relation->hints
-	relation->hints &= ~UG_HINT_ACTIVE;
-//	relation->hints |=  UG_HINT_PAUSED;
+	// added on
+	datalog = ug_dataset_realloc (dataset, UG_DATA_LOG_I, 0);
+	if (datalog->added_on == NULL)
+		datalog->added_on = ug_str_from_time (time (NULL), FALSE);
+
+	category->funcs->add (category, dataset);
 }
 
+// get all tasks(UgDataset) in this category.
+// To free the returned value, use g_list_free (list).
+GList*	ug_category_get_all (UgCategory* category)
+{
+	return category->funcs->get_all (category);
+}
+
+// get queuing tasks(UgDataset) in this category/
+// This function should be noticed UgCategory::active_limit, because
+// application will try to activate all returned dataset.
+// To free the returned value, use g_list_free (list).
+GList*	ug_category_get_tasks (UgCategory* category)
+{
+	return category->funcs->get_tasks (category);
+}
+
+// used to notify category that it's dataset was changed.
+// It may change hints and switch dataset to another internal queue of category.
+void	ug_category_changed  (UgCategory* category, UgDataset* dataset)
+{
+	category->funcs->changed (category, dataset);
+}
+
+
 // ----------------------------------------------------------------------------
 // Category.indices load/save
 //
@@ -263,12 +188,12 @@ static GMarkupParser	ug_int_list_parser =
 	NULL
 };
 
-void	ug_int_list_in_markup (GList** list, GMarkupParseContext* context)
+static void	ug_int_list_in_markup (GList** list, GMarkupParseContext* context)
 {
 	g_markup_parse_context_push (context, &ug_int_list_parser, list);
 }
 
-void	ug_int_list_to_markup (GList** list, UgMarkup* markup)
+static void	ug_int_list_to_markup (GList** list, UgMarkup* markup)
 {
 	GList*		link;
 	guint		value;
@@ -297,11 +222,9 @@ static void ug_category_data_start_element (GMarkupParseContext*	context,
 		g_markup_parse_context_push (context, &ug_markup_skip_parser, NULL);
 		return;
 	}
-	if (UgCategoryClass == NULL)
-		UgCategoryClass = ug_data_class_find (UG_CATEGORY_CLASS_NAME);
 
-	// user must register data class of UgCategory.
-	category = ug_data_new (UgCategoryClass);
+	// user must register data interface of UgCategory.
+	category = ug_data_new (ug_category_iface_pointer);
 	*list = g_list_prepend (*list, category);
 	g_markup_parse_context_push (context, &ug_data_parser, category);
 }
@@ -322,22 +245,22 @@ static void ug_category_list_start_element (GMarkupParseContext*	context,
                                             GList**				list,
                                             GError**			error)
 {
-	guint	index;
+//	guint	index;
 
-	if (strcmp (element_name, "UgetCategoryList") == 0) {
-		for (index=0; attr_names[index]; index++) {
-			if (strcmp (attr_names[index], "version") != 0)
-				continue;
-			if (strcmp (attr_values[index], "1") == 0) {
+//	if (strcmp (element_name, "UgCategoryList") == 0) {
+//		for (index=0; attr_names[index]; index++) {
+//			if (strcmp (attr_names[index], "version") != 0)
+//				continue;
+//			if (strcmp (attr_values[index], "1") == 0) {
 				g_markup_parse_context_push (context, &ug_category_data_parser, list);
 				return;
-			}
-			// others...
-			break;
-		}
-	}
+//			}
+//			// others...
+//			break;
+//		}
+//	}
 
-	g_markup_parse_context_push (context, &ug_markup_skip_parser, NULL);
+//	g_markup_parse_context_push (context, &ug_markup_skip_parser, NULL);
 }
 
 static GMarkupParser	ug_category_list_parser =
@@ -358,8 +281,9 @@ GList*	ug_category_list_load (const gchar* file)
 	return category_list;
 }
 
-gboolean	ug_category_list_save (GList* list, const gchar* file, UgCategoryGetList get_list)
+gboolean	ug_category_list_save (GList* list, const gchar* file)
 {
+	UgCategoryGetAllFunc	get_all;
 	UgCategory*	category;
 	UgMarkup*	markup;
 	GList*		link;
@@ -371,31 +295,33 @@ gboolean	ug_category_list_save (GList* list, const gchar* file, UgCategoryGetLis
 		return FALSE;
 	}
 
-	ug_markup_write_element_start (markup, "UgetCategoryList version='1'");
+	ug_markup_write_element_start (markup, "UgCategoryList version='1'");
 	for (list = g_list_last (list);  list;  list = list->prev) {
 		category = list->data;
+		get_all = category->funcs->get_all;
 		// create UgCategory.indices
-		category->indices = get_list (category);
+		category->indices = get_all (category);
 		for (link = category->indices;  link;  link = link->next) {
 			index = UG_DATASET_RELATION ((UgDataset*) link->data)->index;
 			link->data = GINT_TO_POINTER (index);
 		}
 		// output
 		ug_markup_write_element_start (markup, "category");
-		ug_data_to_markup ((UgData*) list->data, markup);
+		ug_data_write_markup ((UgData*) list->data, markup);
 		ug_markup_write_element_end (markup, "category");
 		// free UgCategory.indices
 		g_list_free (category->indices);
 		category->indices = NULL;
 	}
-	ug_markup_write_element_end (markup, "UgetCategoryList");
+	ug_markup_write_element_end (markup, "UgCategoryList");
 
 	ug_markup_write_end (markup);
 	return	TRUE;
 }
 
-void	ug_category_list_link (GList* list, GList* download_list, UgCategoryAddFunc add_func)
+void	ug_category_list_link (GList* list, GList* download_list)
 {
+	UgCategoryAddFunc	add_func;
 	GPtrArray*	array;
 	UgCategory*	category;
 	GList*		link;
@@ -406,10 +332,11 @@ void	ug_category_list_link (GList* list, GList* download_list, UgCategoryAddFunc
 	for (link = download_list;  link;  link = link->next)
 		array->pdata[array->len++] = link->data;
 
-	// link jobs in category
+	// link tasks in category
 	for (;  list;  list = list->next) {
 		category = list->data;
-		// get jobs from array by index
+		add_func = category->funcs->add;
+		// get tasks from array by index
 		for (link = category->indices;  link;  link = link->next) {
 			index = GPOINTER_TO_INT (link->data);
 			if (index < array->len)
@@ -463,22 +390,22 @@ static void ug_download_list_start_element (GMarkupParseContext*	context,
                                             GList**				list,
                                             GError**			error)
 {
-	guint	index;
+//	guint	index;
 
-	if (strcmp (element_name, "UgetDownloadList") == 0) {
-		for (index=0; attr_names[index]; index++) {
-			if (strcmp (attr_names[index], "version") != 0)
-				continue;
-			if (strcmp (attr_values[index], "1") == 0) {
+//	if (strcmp (element_name, "UgDownloadList") == 0) {
+//		for (index=0; attr_names[index]; index++) {
+//			if (strcmp (attr_names[index], "version") != 0)
+//				continue;
+//			if (strcmp (attr_values[index], "1") == 0) {
 				g_markup_parse_context_push (context, &ug_download_data_parser, list);
 				return;
-			}
+//			}
 			// others...
-			break;
-		}
-	}
+//			break;
+//		}
+//	}
 
-	g_markup_parse_context_push (context, &ug_markup_skip_parser, NULL);
+//	g_markup_parse_context_push (context, &ug_markup_skip_parser, NULL);
 }
 
 static GMarkupParser	ug_download_list_parser =
@@ -493,6 +420,7 @@ static GMarkupParser	ug_download_list_parser =
 GList*	ug_download_list_load (const gchar* download_file)
 {
 	UgDataCommon*	common;
+	UgRelation*		relation;
 	GList*			list;
 	GList*			link;
 
@@ -500,12 +428,23 @@ GList*	ug_download_list_load (const gchar* download_file)
 	ug_markup_parse (download_file, &ug_download_list_parser, &list);
 	// attachment
 	for (link = list;  link;  link = link->next) {
+		relation = UG_DATASET_RELATION ((UgDataset*) link->data);
+
+		// Compatibility code for old DownloadList.xml
+		if (relation == NULL)
+			relation = ug_dataset_alloc_front (link->data, UG_RELATION_I);
 		common = UG_DATASET_COMMON ((UgDataset*) link->data);
-		if (common == NULL)
-			continue;
-		ug_attachment_ref (common->attached_stamp);
+		if (relation->attached.stamp == 0 && common) {
+			relation->attached.stamp = common->attached.stamp;
+			ug_str_set (&relation->attached.folder, common->attached.folder, -1);
+			g_free (common->attached.folder);
+			common->attached.folder = NULL;
+			common->attached.stamp = 0;
+		}
+
+		ug_attachment_ref (relation->attached.stamp);
 	}
-	// return
+
 	return list;
 }
 
@@ -529,13 +468,13 @@ gboolean	ug_download_list_save (GList* list, const gchar* download_file)
 			break;
 	}
 
-	ug_markup_write_element_start (markup, "UgetDownloadList version='1'");
+	ug_markup_write_element_start (markup, "UgDownloadList version='1'");
 	for (;  list;  list = list->prev) {
 		ug_markup_write_element_start (markup, "download");
-		ug_data_to_markup ((UgData*) list->data, markup);
+		ug_data_write_markup ((UgData*) list->data, markup);
 		ug_markup_write_element_end (markup, "download");
 	}
-	ug_markup_write_element_end (markup, "UgetDownloadList");
+	ug_markup_write_element_end (markup, "UgDownloadList");
 
 	ug_markup_write_end (markup);
 	return	TRUE;
@@ -544,50 +483,32 @@ gboolean	ug_download_list_save (GList* list, const gchar* download_file)
 // ----------------------------------------------------------------------------
 // Below utility functions can be used by g_list_foreach()
 
-gboolean	ug_download_attachment_create (UgDataset* dataset)
+gboolean	ug_download_create_attachment (UgDataset* dataset, gboolean force)
 {
 	UgDataHttp*		http;
-	UgDataCommon*	common;
-	gboolean		result;
+	UgRelation*		relation;
 	gchar*			file;
 	gchar*			dir;
 	guint			dir_len;
 
-	result = FALSE;
-	// UgDataCommon
-	common = UG_DATASET_COMMON (dataset);
-	if (common == NULL)
-		common = ug_dataset_alloc_front (dataset, UgDataCommonClass);
-	else if (common->url == NULL && common->file)
-		result = TRUE;
-	// UgDataHttp
-	http = ug_dataset_get (dataset, UgDataHttpClass, 0);
-	if (http && (http->cookie_file || http->post_file))
-		result = TRUE;
+	// check
+	relation = UG_DATASET_RELATION (dataset);
+	if (relation == NULL)
+		relation = ug_dataset_alloc_front (dataset, UG_RELATION_I);
+	else if (relation->attached.stamp)
+		return FALSE;
+	http = ug_dataset_get (dataset, UG_DATA_HTTP_I, 0);
+	if ( (http && (http->cookie_file || http->post_file)) == FALSE && force == FALSE)
+		return FALSE;
 
 	// create attachment folder
-	if (result == FALSE)
-		return FALSE;
-	dir = ug_attachment_alloc (&common->attached_stamp);
+	dir = ug_attachment_alloc (&relation->attached.stamp);
 	if (dir == NULL)
 		return FALSE;
 	dir_len = strlen (dir);
-	g_free (common->attached_folder);
-	common->attached_folder = dir;
+	g_free (relation->attached.folder);
+	relation->attached.folder = dir;
 
-	// UgDataCommon
-	if (common  &&  common->url == NULL  &&  common->file &&
-	    dir_len != strspn (dir, common->file))
-	{
-		file = g_build_filename (dir, "common-file", NULL);
-		// copy file and save path
-		if (ug_copy_file (common->file, file) == -1)
-			g_free (file);
-		else {
-			g_free (common->file);
-			common->file = file;
-		}
-	}
 	// UgDataHttp
 	if (http) {
 		if (http->cookie_file && dir_len != strspn (dir, http->cookie_file))
@@ -617,57 +538,68 @@ gboolean	ug_download_attachment_create (UgDataset* dataset)
 	return TRUE;
 }
 
-gboolean	ug_download_attachment_set (UgDataset* dest_data, UgDataset* src_data)
+gboolean	ug_download_assign_attachment (UgDataset* dest_data, UgDataset* src_data)
 {
 	union {
 		UgDataHttp*		http;
-		UgDataCommon*	common;
+		UgRelation*		relation;
 	} src;
 	union {
 		UgDataHttp*		http;
-		UgDataCommon*	common;
+		UgRelation*		relation;
 	} dest;
 
-	// common
-	src.common = UG_DATASET_COMMON (src_data);
-	if (src.common == NULL || src.common->attached_stamp == 0)
+	// relation
+	src.relation = UG_DATASET_RELATION (src_data);
+	if (src.relation == NULL || src.relation->attached.stamp == 0)
 		return FALSE;
-	dest.common = ug_dataset_realloc (dest_data, UgDataCommonClass, 0);
-	dest.common->attached_stamp = src.common->attached_stamp;
-	ug_str_set (&dest.common->attached_folder, src.common->attached_folder, -1);
-	// reference count
-	ug_attachment_ref (dest.common->attached_stamp);
+	dest.relation = UG_DATASET_RELATION (dest_data);
+	if (dest.relation == NULL)
+		dest.relation = ug_dataset_alloc_front (src_data, UG_RELATION_I);
+	ug_attachment_ref (src.relation->attached.stamp);
+	ug_attachment_unref (dest.relation->attached.stamp);
+	dest.relation->attached.stamp = src.relation->attached.stamp;
+	ug_str_set (&dest.relation->attached.folder, src.relation->attached.folder, -1);
+
 	// http
-	src.http  = ug_dataset_get (src_data, UgDataHttpClass, 0);
+	src.http  = ug_dataset_get (src_data, UG_DATA_HTTP_I, 0);
 	if (src.http) {
-		dest.http = ug_dataset_realloc (dest_data, UgDataHttpClass, 0);
+		dest.http = ug_dataset_realloc (dest_data, UG_DATA_HTTP_I, 0);
 		ug_str_set (&dest.http->post_file,   src.http->post_file,   -1);
 		ug_str_set (&dest.http->cookie_file, src.http->cookie_file, -1);
 	}
 	return TRUE;
 }
 
-void	ug_download_temp_delete (UgDataset* dataset)
+void	ug_download_delete_temp (UgDataset* dataset)
 {
 	UgDataCommon*	common;
 	gchar*			file;
+	gchar*			file_aria2;
 	gchar*			path;
 
 	common = UG_DATASET_COMMON (dataset);
 	if (common && common->file) {
 		file = g_strconcat (common->file, ".ug_", NULL);
-		if (common->folder == NULL)
+		file_aria2 = g_strconcat (common->file, ".aria2", NULL);
+		if (common->folder == NULL) {
 			ug_delete_file (file);
+			ug_delete_file (file_aria2);
+		}
 		else {
 			path = g_build_filename (common->folder, file, NULL);
 			ug_delete_file (path);
 			g_free (path);
+			// aria2 control file
+			path = g_build_filename (common->folder, file_aria2, NULL);
+			ug_delete_file (path);
+			g_free (path);
 		}
 		g_free (file);
 	}
 }
 
-void	ug_download_data_complete (UgDataset* dataset)
+void	ug_download_complete_data (UgDataset* dataset)
 {
 	UgUriFull		urifull;
 	UgDataCommon*	common;
@@ -707,3 +639,62 @@ void	ug_download_data_complete (UgDataset* dataset)
 }
 
 
+// ----------------------------------------------------------------------------
+// UgRelation : relation of UgCategory, UgDataset, and UgPlugin.
+//
+static void	ug_relation_finalize (UgRelation* relation);
+static void	ug_relation_assign   (UgRelation* relation, UgRelation* src);
+
+static const UgDataEntry	ug_relation_entry[] =
+{
+	{"hints",			G_STRUCT_OFFSET (UgRelation, hints),			UG_DATA_UINT,	NULL,	NULL},
+	{"AttachedFolder",	G_STRUCT_OFFSET (UgRelation, attached.folder),	UG_DATA_STRING,	NULL,	NULL},
+	{"AttachedStamp",	G_STRUCT_OFFSET (UgRelation, attached.stamp),	UG_DATA_UINT,	NULL,	NULL},
+	{"MessageType",		G_STRUCT_OFFSET (UgRelation, message.type),		UG_DATA_UINT,	NULL,	NULL},
+	{"MessageString",	G_STRUCT_OFFSET (UgRelation, message.string),	UG_DATA_STRING,	NULL,	NULL},
+	{NULL},			// null-terminated
+};
+// extern
+const UgDataInterface	ug_relation_iface =
+{
+	sizeof (UgRelation),	// instance_size
+	"relation",				// name
+	ug_relation_entry,		// entry
+
+	(UgInitFunc)     NULL,
+	(UgFinalizeFunc) ug_relation_finalize,
+	(UgAssignFunc)   ug_relation_assign,
+};
+// extern
+const UgDataInterface*	ug_relation_iface_pointer = &ug_relation_iface;
+
+
+static void	ug_relation_finalize (UgRelation* relation)
+{
+	if (relation->destroy.func)
+		relation->destroy.func (relation->destroy.data);
+
+	g_free (relation->attached.folder);
+	ug_attachment_unref (relation->attached.stamp);
+}
+
+static void	ug_relation_assign (UgRelation* relation, UgRelation* src)
+{
+	// hints
+	// used by UgDownloadDialog
+	if (src->hints & UG_HINT_PAUSED) {
+		relation->hints |=  UG_HINT_PAUSED;
+		relation->hints &= ~UG_HINT_ACTIVE;
+	}
+	else
+		relation->hints &= ~UG_HINT_PAUSED;
+
+	// attachment
+	if (relation->attached.stamp != src->attached.stamp) {
+		ug_attachment_ref (src->attached.stamp);
+		ug_attachment_unref (relation->attached.stamp);
+		relation->attached.stamp = src->attached.stamp;
+		ug_str_set (&relation->attached.folder, src->attached.folder, -1);
+	}
+}
+
