@@ -1,7 +1,7 @@
 /*
  *
- *   Copyright (C) 2005-2010 by Raymond Huang
- *   plushuang at users.sourceforge.net
+ *   Copyright (C) 2005-2011 by C.H. Huang
+ *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -115,11 +115,11 @@ GList*	ug_option_parse (UgOption* uopt, GPtrArray* args)
 	strings = g_list_concat (strings, ug_arg_get_rest (temp_args));
 	ug_arg_free (temp_args, FALSE);
 
-	// get argument data and create jobs
+	// get argument data and create tasks
 	downloads = NULL;
 	for (strings = g_list_last (strings);  strings;  strings = strings->prev) {
 		dataset = ug_dataset_new ();
-		common = ug_dataset_alloc_front (dataset, UgDataCommonClass);
+		common = ug_dataset_alloc_front (dataset, UG_DATA_COMMON_I);
 		common->url = strings->data;
 		common->keeping.url = TRUE;		// keep common->url no change
 		g_list_foreach (uopt->list, (GFunc) ug_option_entry_get, dataset);
@@ -162,7 +162,7 @@ void ug_option_entry_clear (const UgOptionEntry* oentry)
 static UgOptionMainData		option_main_data;
 static GOptionEntry			option_main_entry[] =
 {
-	{"version",		  'V',	0,	G_OPTION_ARG_NONE,		&option_main_data.version,			"display the version of Uget and exit.",		NULL},
+	{"version",		  'V',	0,	G_OPTION_ARG_NONE,		&option_main_data.version,			"display the version of uGet and exit.",		NULL},
 	{"set-offline",		0,	0,	G_OPTION_ARG_INT,		&option_main_data.offline,			"set offline mode to N. (0=Disable)",			"N"},
 
 	{"quiet",			0,	0,	G_OPTION_ARG_NONE,		&option_main_data.quiet,			"add download directly. Don't show dialog.",	NULL},
@@ -220,7 +220,7 @@ static void	ug_option_main_entry_init	(UgOptionMainData* option_data)
 static gboolean	ug_option_main_entry_get (UgOptionEntry* oentry, UgDataset* dataset)
 {
 	UgOptionMainData*	option_data;
-	gboolean		result = FALSE;
+	gboolean		retval = FALSE;
 	UgDataCommon*	common;
 	UgDataProxy*	proxy;
 	UgDataHttp*		http;
@@ -229,159 +229,159 @@ static gboolean	ug_option_main_entry_get (UgOptionEntry* oentry, UgDataset* data
 	option_data = oentry->data;
 	// common
 	if (option_data->folder) {
-		common = ug_dataset_realloc (dataset, UgDataCommonClass, 0);
+		common = ug_dataset_realloc (dataset, UG_DATA_COMMON_I, 0);
 		g_free (common->folder);
 		common->folder = g_strdup (option_data->folder);
 		common->keeping.folder = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->file) {
-		common = ug_dataset_realloc (dataset, UgDataCommonClass, 0);
+		common = ug_dataset_realloc (dataset, UG_DATA_COMMON_I, 0);
 		g_free (common->file);
 		common->file = g_strdup (option_data->file);
 		common->keeping.file = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->user) {
-		common = ug_dataset_realloc (dataset, UgDataCommonClass, 0);
+		common = ug_dataset_realloc (dataset, UG_DATA_COMMON_I, 0);
 		g_free (common->user);
 		common->user = g_strdup (option_data->user);
 		common->keeping.user = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->password) {
-		common = ug_dataset_realloc (dataset, UgDataCommonClass, 0);
+		common = ug_dataset_realloc (dataset, UG_DATA_COMMON_I, 0);
 		g_free (common->password);
 		common->password = g_strdup (option_data->password);
 		common->keeping.password = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	// proxy
 	if (option_data->proxy.type) {
-		proxy = ug_dataset_realloc (dataset, UgDataProxyClass, 0);
+		proxy = ug_dataset_realloc (dataset, UG_DATA_PROXY_I, 0);
 		proxy->type = option_data->proxy.type;
 		proxy->keeping.type = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->proxy.host) {
-		proxy = ug_dataset_realloc (dataset, UgDataProxyClass, 0);
+		proxy = ug_dataset_realloc (dataset, UG_DATA_PROXY_I, 0);
 		g_free (proxy->host);
 		proxy->host = g_strdup (option_data->proxy.host);
 		proxy->keeping.host = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->proxy.port) {
-		proxy = ug_dataset_realloc (dataset, UgDataProxyClass, 0);
+		proxy = ug_dataset_realloc (dataset, UG_DATA_PROXY_I, 0);
 		proxy->port = option_data->proxy.port;
 		proxy->keeping.port = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->proxy.user) {
-		proxy = ug_dataset_realloc (dataset, UgDataProxyClass, 0);
+		proxy = ug_dataset_realloc (dataset, UG_DATA_PROXY_I, 0);
 		g_free (proxy->user);
 		proxy->user = g_strdup (option_data->proxy.user);
 		proxy->keeping.user = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->proxy.password) {
-		proxy = ug_dataset_realloc (dataset, UgDataProxyClass, 0);
+		proxy = ug_dataset_realloc (dataset, UG_DATA_PROXY_I, 0);
 		g_free (proxy->password);
 		proxy->password = g_strdup (option_data->proxy.password);
 		proxy->keeping.password = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	// http
 	if (option_data->http.user) {
-		http = ug_dataset_realloc (dataset, UgDataHttpClass, 0);
+		http = ug_dataset_realloc (dataset, UG_DATA_HTTP_I, 0);
 		g_free (http->user);
 		http->user = g_strdup (option_data->http.user);
 		http->keeping.user = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->http.password) {
-		http = ug_dataset_realloc (dataset, UgDataHttpClass, 0);
+		http = ug_dataset_realloc (dataset, UG_DATA_HTTP_I, 0);
 		g_free (http->password);
 		http->password = g_strdup (option_data->http.password);
 		http->keeping.password = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->http.referrer) {
-		http = ug_dataset_realloc (dataset, UgDataHttpClass, 0);
+		http = ug_dataset_realloc (dataset, UG_DATA_HTTP_I, 0);
 		g_free (http->referrer);
 		http->referrer = g_strdup (option_data->http.referrer);
 		http->keeping.referrer = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->http.user_agent) {
-		http = ug_dataset_realloc (dataset, UgDataHttpClass, 0);
+		http = ug_dataset_realloc (dataset, UG_DATA_HTTP_I, 0);
 		g_free (http->user_agent);
 		http->user_agent = g_strdup (option_data->http.user_agent);
 		http->keeping.user_agent = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->http.cookie_data) {
-		http = ug_dataset_realloc (dataset, UgDataHttpClass, 0);
+		http = ug_dataset_realloc (dataset, UG_DATA_HTTP_I, 0);
 		g_free (http->cookie_data);
 		http->cookie_data = g_strdup (option_data->http.cookie_data);
 		http->keeping.cookie_data = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->http.cookie_file) {
-		http = ug_dataset_realloc (dataset, UgDataHttpClass, 0);
+		http = ug_dataset_realloc (dataset, UG_DATA_HTTP_I, 0);
 		g_free (http->cookie_file);
 		http->cookie_file = g_strdup (option_data->http.cookie_file);
 		http->keeping.cookie_file = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->http.post_data) {
-		http = ug_dataset_realloc (dataset, UgDataHttpClass, 0);
+		http = ug_dataset_realloc (dataset, UG_DATA_HTTP_I, 0);
 		g_free (http->post_data);
 		http->post_data = g_strdup (option_data->http.post_data);
 		http->keeping.post_data = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->http.post_file) {
-		http = ug_dataset_realloc (dataset, UgDataHttpClass, 0);
+		http = ug_dataset_realloc (dataset, UG_DATA_HTTP_I, 0);
 		g_free (http->post_file);
 		http->post_file = g_strdup (option_data->http.post_file);
 		http->keeping.post_file = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	// ftp
 	if (option_data->ftp.user) {
-		ftp = ug_dataset_realloc (dataset, UgDataFtpClass, 0);
+		ftp = ug_dataset_realloc (dataset, UG_DATA_FTP_I, 0);
 		g_free (ftp->user);
 		ftp->user = g_strdup (option_data->ftp.user);
 		ftp->keeping.user = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
 	if (option_data->ftp.password) {
-		ftp = ug_dataset_realloc (dataset, UgDataFtpClass, 0);
+		ftp = ug_dataset_realloc (dataset, UG_DATA_FTP_I, 0);
 		g_free (ftp->password);
 		ftp->password = g_strdup (option_data->ftp.password);
 		ftp->keeping.password = TRUE;
-		result = TRUE;
+		retval = TRUE;
 	}
 
-	return result;
+	return retval;
 }
 
 // ------------------------------------
