@@ -1,7 +1,7 @@
 /*
  *
- *   Copyright (C) 2005-2010 by Raymond Huang
- *   plushuang at users.sourceforge.net
+ *   Copyright (C) 2005-2011 by C.H. Huang
+ *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -47,9 +47,9 @@ extern "C" {
 #endif
 
 
-typedef struct	UgOption_			UgOption;
-typedef struct	UgOptionEntry_		UgOptionEntry;
-typedef struct	UgOptionMainData_	UgOptionMainData;
+typedef struct	UgOption			UgOption;
+typedef struct	UgOptionEntry		UgOptionEntry;
+typedef struct	UgOptionMainData	UgOptionMainData;
 
 typedef gboolean	(*UgOptionEntryGet)	(const UgOptionEntry* oentry, UgDataset* dataset);
 
@@ -57,7 +57,7 @@ typedef gboolean	(*UgOptionEntryGet)	(const UgOptionEntry* oentry, UgDataset* da
 // ----------------------------------------------------------------------------
 // UgOption: integrate GOptionContext, GOptionGroup, and UgOptionEntry
 //
-struct	UgOption_
+struct	UgOption
 {
 	GOptionContext*		context;
 	GOptionGroup*		group;		// main group
@@ -85,7 +85,7 @@ const UgOptionEntry*	ug_option_get_main_entry (void);
 // ----------------------------------------------------------------------------
 // UgOptionEntry
 //
-struct UgOptionEntry_
+struct UgOptionEntry
 {
 	const gchar*		name;
 	gpointer			reserve;			// reserve for GModule-related code
@@ -108,7 +108,7 @@ void	ug_option_entry_clear (const UgOptionEntry* oentry);
 // ---------------------------------------------------------------------------
 // UgOptionMainData: main argument data
 //
-struct UgOptionMainData_
+struct UgOptionMainData
 {
 	// option_entry->arg_data place below
 	gboolean	version;
@@ -124,7 +124,8 @@ struct UgOptionMainData_
 	gchar*		user;
 	gchar*		password;
 
-	struct {
+	struct
+	{
 		gint		type;
 		gchar*		host;
 		guint		port;
@@ -132,7 +133,8 @@ struct UgOptionMainData_
 		gchar*		password;
 	} proxy;
 
-	struct {
+	struct
+	{
 		gchar*		user;
 		gchar*		password;
 		gchar*		referrer;
@@ -143,7 +145,8 @@ struct UgOptionMainData_
 		gchar*		post_file;
 	} http;
 
-	struct {
+	struct
+	{
 		gchar*		user;
 		gchar*		password;
 	} ftp;
