@@ -1,7 +1,7 @@
 /*
  *
- *   Copyright (C) 2005-2010 by Raymond Huang
- *   plushuang at users.sourceforge.net
+ *   Copyright (C) 2005-2011 by C.H. Huang
+ *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -33,14 +33,14 @@
  *  files in the program, then also delete it here.
  *
  */
-#ifdef _WIN32
-#include <sys/utime.h>
-#else
-#include <utime.h>
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
 #endif
 
+#ifdef HAVE_PLUGIN_CURL
+
 #include <string.h>
-#include <glib/gstdio.h>
 #include <UgPlugin-curl.h>
 #include <UgData-download.h>
 #include <UgStdio.h>
@@ -49,8 +49,7 @@
 #include <UgUri.h>
 
 #ifdef HAVE_LIBPWMD
-#include <stdlib.h>
-#include <libpwmd.h>
+#include "pwmd.h"
 static gboolean	ug_plugin_curl_set_proxy_pwmd (UgPluginCurl *plugin);
 #endif  // HAVE_LIBPWMD
 
@@ -78,11 +77,11 @@ typedef size_t (*curl_read_callback)(char *buffer,
                                      void *instream);
 */
 
-// functions for UgPluginClass
+// functions for UgPluginInterface
 static gboolean	ug_plugin_curl_global_init		(void);
 static void		ug_plugin_curl_global_finalize	(void);
 
-static void		ug_plugin_curl_init			(UgPluginCurl* plugin, UgDataset* dataset);
+static gboolean	ug_plugin_curl_init			(UgPluginCurl* plugin, UgDataset* dataset);
 static void		ug_plugin_curl_finalize		(UgPluginCurl* plugin);
 
 static UgResult	ug_plugin_curl_set_state	(UgPluginCurl* plugin, UgState  state);
@@ -100,22 +99,21 @@ static void		ug_plugin_curl_set_proxy	(UgPluginCurl* plugin, CURL* curl);
 static int		ug_plugin_curl_progress		(UgPluginCurl* plugin, double  dltotal, double  dlnow, double  ultotal, double  ulnow);
 static size_t	ug_plugin_curl_header_http	(char *buffer, size_t size, size_t nmemb, UgPluginCurl *plugin);
 
-// static functions
-static int		ug_set_file_time (const gchar *file_utf8, time_t mod_time);
-
-// static data for UgPluginClass
+// static data for UgPluginInterface
 static const char*	supported_schemes[] = {"http", "https", "ftp", "ftps", NULL};
-
-static const	UgPluginClass	plugin_class_curl =
+// extern
+const	UgPluginInterface	ug_plugin_curl_iface =
 {
-	"curl",														// name
-	NULL,														// reserve
 	sizeof (UgPluginCurl),										// instance_size
+	"curl",														// name
+
 	supported_schemes,											// schemes
 	NULL,														// file_types
 
 	(UgGlobalInitFunc)		ug_plugin_curl_global_init,			// global_init
 	(UgGlobalFinalizeFunc)	ug_plugin_curl_global_finalize,		// global_finalize
+	(UgGlobalSetFunc)		NULL,								// global_set
+	(UgGlobalGetFunc)		NULL,								// global_get
 
 	(UgPluginInitFunc)		ug_plugin_curl_init,				// init
 	(UgFinalizeFunc)		ug_plugin_curl_finalize,			// finalize
@@ -126,11 +124,9 @@ static const	UgPluginClass	plugin_class_curl =
 	(UgGetFunc)				ug_plugin_curl_get,					// get
 };
 
-// extern
-const UgPluginClass*	UgPluginCurlClass = &plugin_class_curl;
 
 // ----------------------------------------------------------------------------
-// functions for UgPluginClass
+// functions for UgPluginInterface
 static gboolean	ug_plugin_curl_global_init (void)
 {
 	curl_global_init (CURL_GLOBAL_ALL);
@@ -142,42 +138,44 @@ static void	ug_plugin_curl_global_finalize (void)
 	curl_global_cleanup ();
 }
 
-static void	ug_plugin_curl_init (UgPluginCurl* plugin, UgDataset* dataset)
+static gboolean	ug_plugin_curl_init (UgPluginCurl* plugin, UgDataset* dataset)
 {
 	UgDataCommon*	common;
 	UgDataHttp*		http;
 
-	common = ug_dataset_get (dataset, UgDataCommonClass, 0);
-	http   = ug_dataset_get (dataset, UgDataHttpClass, 0);
+	common = ug_dataset_get (dataset, UG_DATA_COMMON_I, 0);
+	http   = ug_dataset_get (dataset, UG_DATA_HTTP_I, 0);
+	// check data
+	if (common == NULL  ||  common->url == NULL)
+		return FALSE;
 	// reset
 	if (common)
 		common->retry_count = 0;
 	if (http)
 		http->redirection_count = 0;
 	// copy supported data
-	plugin->common = ug_data_list_copy (ug_dataset_get (dataset, UgDataCommonClass, 0));
-	plugin->proxy  = ug_data_list_copy (ug_dataset_get (dataset, UgDataProxyClass, 0));
-	plugin->http   = ug_data_list_copy (ug_dataset_get (dataset, UgDataHttpClass, 0));
-	plugin->ftp    = ug_data_list_copy (ug_dataset_get (dataset, UgDataFtpClass, 0));
-
-	// initialize this struct/class
-//	plugin->output_func = dataset->output_func;
-//	plugin->output_data = dataset->output_data;
+	plugin->common = ug_datalist_copy (ug_dataset_get (dataset, UG_DATA_COMMON_I, 0));
+	plugin->proxy  = ug_datalist_copy (ug_dataset_get (dataset, UG_DATA_PROXY_I, 0));
+	plugin->http   = ug_datalist_copy (ug_dataset_get (dataset, UG_DATA_HTTP_I, 0));
+	plugin->ftp    = ug_datalist_copy (ug_dataset_get (dataset, UG_DATA_FTP_I, 0));
 
 	// default values
 	plugin->resumable = TRUE;
+	// initial curl
+	plugin->curl = curl_easy_init ();
+
+	return TRUE;
 }
 
 static void	ug_plugin_curl_finalize (UgPluginCurl* plugin)
 {
 	// free data
-	ug_data_list_free (plugin->common);
-	ug_data_list_free (plugin->proxy);
-	ug_data_list_free (plugin->http);
-	ug_data_list_free (plugin->ftp);
+	ug_datalist_free (plugin->common);
+	ug_datalist_free (plugin->proxy);
+	ug_datalist_free (plugin->http);
+	ug_datalist_free (plugin->ftp);
 	// clear curl
-	if (plugin->curl)
-		curl_easy_cleanup (plugin->curl);
+	curl_easy_cleanup (plugin->curl);
 }
 
 static UgResult	ug_plugin_curl_set_state (UgPluginCurl* plugin, UgState  state)
@@ -209,9 +207,18 @@ static UgResult	ug_plugin_curl_get_state (UgPluginCurl* plugin, UgState* state)
 	return UG_RESULT_ERROR;
 }
 
-UgResult	ug_plugin_curl_set (UgPluginCurl* plugin, guint parameter, gpointer data)
+static UgResult	ug_plugin_curl_set (UgPluginCurl* plugin, guint parameter, gpointer data)
 {
-	return UG_RESULT_UNSUPPORT;
+	gint64	speed_limit;
+
+	if (parameter != UG_DATA_INT64)
+		return UG_RESULT_UNSUPPORT;
+
+	speed_limit = *(gint64*)data;
+	curl_easy_setopt (plugin->curl, CURLOPT_MAX_RECV_SPEED_LARGE, (curl_off_t) speed_limit);
+	curl_easy_setopt (plugin->curl, CURLOPT_MAX_SEND_SPEED_LARGE, (curl_off_t) speed_limit);
+
+	return UG_RESULT_OK;
 }
 
 static UgResult	ug_plugin_curl_get (UgPluginCurl* plugin, guint parameter, gpointer data)
@@ -219,16 +226,18 @@ static UgResult	ug_plugin_curl_get (UgPluginCurl* plugin, guint parameter, gpoin
 	UgProgress*	progress;
 	gdouble		complete;
 	gdouble		total;
+	gdouble		speed;
 
-	if (parameter != UG_DATA_TYPE_INSTANCE)
+	if (parameter != UG_DATA_INSTANCE)
 		return UG_RESULT_UNSUPPORT;
-	if (data == NULL || UG_DATA_CAST (data)->data_class != UgProgressClass)
+	if (data == NULL || ((UgData*)data)->iface != UG_PROGRESS_I)
 		return UG_RESULT_UNSUPPORT;
 
 	progress = data;
 	curl_easy_getinfo (plugin->curl, CURLINFO_TOTAL_TIME, &progress->consume_time);
-	curl_easy_getinfo (plugin->curl, CURLINFO_SPEED_DOWNLOAD, &progress->download_speed);
 	curl_easy_getinfo (plugin->curl, CURLINFO_SIZE_DOWNLOAD, &complete);
+	curl_easy_getinfo (plugin->curl, CURLINFO_SPEED_DOWNLOAD, &speed);
+	progress->download_speed = (gint64) speed;
 	complete = plugin->file_beg + complete;
 	total    = plugin->file_end;
 	// if total size is unknown, use completed size.
@@ -241,7 +250,7 @@ static UgResult	ug_plugin_curl_get (UgPluginCurl* plugin, guint parameter, gpoin
 		progress->percent = 0;
 	// If total size and average speed is unknown, don't calculate remain time.
 	if (progress->download_speed > 0 && total > 0)
-		progress->remain_time = (total - complete) / progress->download_speed;
+		progress->remain_time = (gdouble) ((total - complete) / progress->download_speed);
 	// others
 	progress->complete = (gint64) complete;
 	progress->total    = (gint64) total;
@@ -263,31 +272,20 @@ static gpointer	ug_plugin_curl_thread (UgPluginCurl* plugin)
 	UgDataHttp*		http;
 	UgDataFtp*		ftp;
 
+	// supported data
 	common = plugin->common;
 	proxy = plugin->proxy;
 	http = plugin->http;
 	ftp = plugin->ftp;
-	// check data
-	if (common == NULL  ||  common->url == NULL) {
-		ug_plugin_post ((UgPlugin*) plugin,
-				ug_message_new_error (UG_MESSAGE_ERROR_UNSUPPORTED_SCHEME, NULL));
-		return NULL;
-	}
 	// curl
-	if (plugin->curl == NULL)
-		plugin->curl = curl_easy_init ();
 	curl = plugin->curl;
 	curl_code = CURLE_FAILED_INIT;
 
 	// Common option ----------------------------------------------------------
 	curl_easy_setopt (curl, CURLOPT_URL, common->url);
+	curl_easy_setopt (curl, CURLOPT_NOSIGNAL, 1);
 	// setup scheme and it's related data (user & password)
 	ug_plugin_curl_set_scheme (plugin, curl);
-	// speed
-	if (common->speed_limit) {
-		curl_easy_setopt (curl, CURLOPT_MAX_RECV_SPEED_LARGE,
-				(curl_off_t) common->speed_limit);
-	}
 	// check filename
 	if (common->file == NULL) {
 		common->file = ug_uri_get_filename (common->url);
@@ -296,6 +294,12 @@ static gpointer	ug_plugin_curl_thread (UgPluginCurl* plugin)
 		ug_plugin_post ((UgPlugin*) plugin,
 				ug_message_new_data (UG_MESSAGE_DATA_FILE_CHANGED, common->file));
 	}
+	if (common->max_download_speed) {
+		curl_easy_setopt (curl, CURLOPT_MAX_SEND_SPEED_LARGE,
+				(curl_off_t) common->max_upload_speed);
+		curl_easy_setopt (curl, CURLOPT_MAX_RECV_SPEED_LARGE,
+				(curl_off_t) common->max_download_speed);
+	}
 
 	// Proxy option -----------------------------------------------------------
 	if (proxy) {
@@ -375,6 +379,11 @@ static gpointer	ug_plugin_curl_thread (UgPluginCurl* plugin)
 				curl_easy_setopt (curl, CURLOPT_READFUNCTION , fread);
 #endif
 			}
+			else {
+				ug_plugin_post ((UgPlugin*) plugin,
+						ug_message_new_error (UG_MESSAGE_ERROR_HTTP_OPEN_POSTED_FILE_FAILED, NULL));
+				goto exit;
+			}
 		}
 	}
 
@@ -405,7 +414,7 @@ static gpointer	ug_plugin_curl_thread (UgPluginCurl* plugin)
 	// Others -----------------------------------------------------------------
 	curl_easy_setopt (curl, CURLOPT_FILETIME, 1);	// use with CURLINFO_FILETIME
 	curl_easy_setopt (curl, CURLOPT_ERRORBUFFER, plugin->curl_error_string);
-	curl_easy_setopt (curl, CURLOPT_LOW_SPEED_LIMIT, 1000);
+	curl_easy_setopt (curl, CURLOPT_LOW_SPEED_LIMIT, 100);
 	curl_easy_setopt (curl, CURLOPT_LOW_SPEED_TIME, 60);
 
 	while (plugin->state == UG_STATE_ACTIVE) {
@@ -473,7 +482,7 @@ static gpointer	ug_plugin_curl_thread (UgPluginCurl* plugin)
 			ug_fs_truncate (plugin->file_stream, 0);
 			// send message
 			ug_plugin_post ((UgPlugin*) plugin,
-					ug_message_new_info (UG_MESSAGE_INFO_UNRESUMABLE, NULL));
+					ug_message_new_info (UG_MESSAGE_INFO_NOT_RESUMABLE, NULL));
 			break;
 
 		// retry
@@ -560,7 +569,7 @@ exit:
 #endif
 		curl_easy_getinfo (curl, CURLINFO_FILETIME, &curl_time);
 		if (curl_time != -1)
-			ug_set_file_time (plugin->path, (time_t) curl_time);
+			ug_modify_file_time (plugin->path, (time_t) curl_time);
 		ug_plugin_post ((UgPlugin*) plugin,
 				ug_message_new_info (UG_MESSAGE_INFO_COMPLETE, NULL));
 		ug_plugin_post ((UgPlugin*) plugin,
@@ -746,8 +755,22 @@ static size_t	ug_plugin_curl_header_http (char *buffer, size_t size, size_t nmem
 	g_print ("%s", buffer);
 #endif
 
+	// handle HTTP header "Accept-Ranges:"
+	if (g_ascii_strncasecmp (buffer, "Accept-Ranges: ", 15) == 0) {
+		buffer += 15;
+		if (strncmp (buffer, "none", 4) == 0) {
+			plugin->resumable = FALSE;
+			ug_plugin_post ((UgPlugin*) plugin,
+					ug_message_new_info (UG_MESSAGE_INFO_NOT_RESUMABLE, NULL));
+		}
+		else {
+			plugin->resumable = TRUE;
+			ug_plugin_post ((UgPlugin*) plugin,
+					ug_message_new_info (UG_MESSAGE_INFO_RESUMABLE, NULL));
+		}
+	}
 	// handle HTTP header "Location:"
-	if (g_ascii_strncasecmp (buffer, "Location: ", 10) == 0) {
+	else if (g_ascii_strncasecmp (buffer, "Location: ", 10) == 0) {
 		// exclude header and character '\r', '\n'
 		buffer += 10;
 		g_free (common->url);
@@ -814,151 +837,57 @@ static size_t	ug_plugin_curl_header_http (char *buffer, size_t size, size_t nmem
 	return nmemb;
 }
 
-// static function
-static int	ug_set_file_time (const gchar *file_utf8, time_t mod_time)
-{
-	struct utimbuf	utb;
-	gchar*			file;
-	int				result;
-
-	utb.actime = time (NULL);
-	utb.modtime = mod_time;
-	file = g_filename_from_utf8 (file_utf8, -1, NULL, NULL, NULL);
-	result = g_utime (file, &utb);
-	g_free (file);
-
-	return result;
-}
-
 // ----------------------------------------------------------------------------
 // PWMD
 //
 #ifdef HAVE_LIBPWMD
 static gboolean	ug_plugin_curl_set_proxy_pwmd (UgPluginCurl *plugin)
 {
-    gpg_error_t rc;
-    gchar *result, *hostname = NULL, *username = NULL, *password = NULL,
-	  *type = NULL;
-    gint port = 80;
-    gchar *path = NULL;
-    gint i;
-    UgMessage *message;
+       UgMessage *message;
+       struct pwmd_proxy_s pwmd;
+       gpg_error_t rc;
 
-    if (plugin->proxy->pwmd.element) {
-	path = g_strdup_printf("%s\t", plugin->proxy->pwmd.element);
+       memset(&pwmd, 0, sizeof(pwmd));
+       rc = ug_set_pwmd_proxy_options(&pwmd, plugin->proxy);
 
-	for (i = 0; i < strlen(path); i++) {
-	    if (path[i] == '^')
-		path[i] = '\t';
-	}
-    }
+       if (rc)
+               goto fail;
 
-    pwmd_init();
-    pwm_t *pwm = pwmd_new("uget");
-    rc = pwmd_connect(pwm, plugin->proxy->pwmd.socket);
+       // proxy host and port
+       curl_easy_setopt (plugin->curl, CURLOPT_PROXY, pwmd.hostname);
+       curl_easy_setopt (plugin->curl, CURLOPT_PROXYPORT, pwmd.port);
 
-    if (rc)
-	goto fail;
+       // proxy user and password
+       if (pwmd.username || pwmd.password || !strcasecmp(pwmd.type, "socks4") || !strcasecmp(pwmd.type, "socks5")) {
+               curl_easy_setopt (plugin->curl, CURLOPT_PROXYUSERNAME,
+                               (pwmd.username) ? pwmd.username : "");
+               curl_easy_setopt (plugin->curl, CURLOPT_PROXYPASSWORD,
+                               (pwmd.password) ? pwmd.password : "");
+       }
+       else {
+               curl_easy_setopt (plugin->curl, CURLOPT_PROXYUSERNAME, NULL);
+               curl_easy_setopt (plugin->curl, CURLOPT_PROXYPASSWORD, NULL);
+       }
 
-    rc = pwmd_open(pwm, plugin->proxy->pwmd.file);
+       if (!strcasecmp(pwmd.type, "socks4"))
+               curl_easy_setopt (plugin->curl, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS4);
+       else if (!strcasecmp(pwmd.type, "socks5"))
+               curl_easy_setopt (plugin->curl, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS5);
 
-    if (rc)
-	goto fail;
-
-    rc = pwmd_command(pwm, &result, "GET %stype", path ? path : "");
-
-    if (rc)
-	goto fail;
-
-    type = result;
-    rc = pwmd_command(pwm, &result, "GET %shostname", path ? path : "");
-
-    if (rc)
-	goto fail;
-
-    hostname = result;
-    rc = pwmd_command(pwm, &result, "GET %sport", path ? path : "");
-
-    if (rc && rc != GPG_ERR_ELEMENT_NOT_FOUND)
-	goto fail;
-
-    port = atoi(result);
-    pwmd_free(result);
-    rc = pwmd_command(pwm, &result, "GET %susername", path ? path : "");
-
-    if (rc && rc != GPG_ERR_ELEMENT_NOT_FOUND)
-	goto fail;
-
-    if (!rc)
-	username = result;
-
-    rc = pwmd_command(pwm, &result, "GET %spassword", path ? path : "");
-
-    if (rc && rc != GPG_ERR_ELEMENT_NOT_FOUND)
-	goto fail;
-
-    if (!rc)
-	password = result;
-
-    // proxy host and port
-    curl_easy_setopt (plugin->curl, CURLOPT_PROXY, hostname);
-    curl_easy_setopt (plugin->curl, CURLOPT_PROXYPORT, port);
-    // proxy user and password
-    if (username || password || !strcasecmp(type, "socks4") || !strcasecmp(type, "socks5")) {
-	curl_easy_setopt (plugin->curl, CURLOPT_PROXYUSERNAME, (username) ? username : "");
-	curl_easy_setopt (plugin->curl, CURLOPT_PROXYPASSWORD, (password) ? password : "");
-    }
-    else {
-	curl_easy_setopt (plugin->curl, CURLOPT_PROXYUSERNAME, NULL);
-	curl_easy_setopt (plugin->curl, CURLOPT_PROXYPASSWORD, NULL);
-    }
-
-    if (!strcasecmp(type, "socks4"))
-	curl_easy_setopt (plugin->curl, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS4);
-    else if (!strcasecmp(type, "socks5"))
-	curl_easy_setopt (plugin->curl, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS5);
-
-    pwmd_free(type);
-    pwmd_free(hostname);
-
-    if (username)
-	pwmd_free(username);
-
-    if (password)
-	pwmd_free(password);
-
-    if (path)
-	g_free(path);
-
-    pwmd_close(pwm);
-    return TRUE;
+       ug_close_pwmd(&pwmd);
+       return TRUE;
 
 fail:
-    if (type)
-	pwmd_free(type);
-
-    if (hostname)
-	pwmd_free(hostname);
-
-    if (username)
-	pwmd_free(username);
-
-    if (password)
-	pwmd_free(password);
-
-    if (path)
-	g_free(path);
-
-    if (pwm)
-	pwmd_close(pwm);
-
-    gchar *e = g_strdup_printf("Pwmd ERR %i: %s", rc, pwmd_strerror(rc));
-    message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, e);
-    ug_plugin_post ((UgPlugin*) plugin, message);
-    fprintf(stderr, "%s\n", e);
-    g_free(e);
-    return FALSE;
+       ug_close_pwmd(&pwmd);
+       gchar *e = g_strdup_printf("Pwmd ERR %i: %s", rc, pwmd_strerror(rc));
+       message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, e);
+       ug_plugin_post ((UgPlugin*) plugin, message);
+       fprintf(stderr, "%s\n", e);
+       g_free(e);
+       return FALSE;
 }
 
 #endif	// HAVE_LIBPWMD
 
+#endif	// HAVE_PLUGIN_CURL
+
