@@ -1,7 +1,7 @@
 /*
  *
- *   Copyright (C) 2005-2010 by Raymond Huang
- *   plushuang at users.sourceforge.net
+ *   Copyright (C) 2005-2011 by C.H. Huang
+ *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -146,7 +146,7 @@ int		ug_fd_truncate (int fd, gint64 length);
 #  define  ug_fd_read		read
 #  define  ug_fd_write		write
 #  define  ug_fd_seek		lseek
-#  define  ug_fd_tell		tell
+#  define  ug_fd_tell(fd)	lseek(fd, 0L, SEEK_CUR)
 #  define  ug_fd_truncate	ftruncate
 #endif
 
