@@ -1,7 +1,7 @@
 /*
  *
- *   Copyright (C) 2005-2010 by Raymond Huang
- *   plushuang at users.sourceforge.net
+ *   Copyright (C) 2005-2011 by C.H. Huang
+ *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -126,11 +126,11 @@ exit:
  */
 unsigned int ug_str_line_len (const char* string, int string_len, unsigned int offset)
 {
-	unsigned int result;
+	unsigned int retval;
 
-	result = ug_str_find_charset(string, string_len, offset, "\r\n");
-	if (result > offset)
-		return result - offset;
+	retval = ug_str_find_charset(string, string_len, offset, "\r\n");
+	if (retval > offset)
+		return retval - offset;
 	return 0;
 }
 
@@ -228,3 +228,30 @@ gchar*	ug_str_from_seconds (guint seconds, gboolean limit_99_99_99)
 	}
 }
 
+/*
+ * convert time_t to string
+ */
+gchar*	ug_str_from_time (time_t ptt, gboolean date_only)
+{
+	GString*	gstr;
+	struct tm*	timem;
+
+	timem = localtime (&ptt);
+	if (date_only)
+		gstr = g_string_sized_new (11);
+	else
+		gstr = g_string_sized_new (11 + 9);
+	g_string_append_printf (gstr, "%.4d-%.2d-%.2d",
+			timem->tm_year + 1900,
+			timem->tm_mon  + 1,
+			timem->tm_mday);
+	if (date_only == FALSE) {
+		g_string_append_printf (gstr, " %.2d:%.2d:%.2d",
+				timem->tm_hour,
+				timem->tm_min,
+				timem->tm_sec);
+	}
+
+	return g_string_free (gstr, FALSE);
+}
+
