@@ -1,7 +1,7 @@
 /*
  *
- *   Copyright (C) 2005-2010 by Raymond Huang
- *   plushuang at users.sourceforge.net
+ *   Copyright (C) 2005-2011 by C.H. Huang
+ *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -63,8 +63,8 @@ extern "C" {
 // |
 // +- UgUriFull
 //
-typedef struct UgUriPart_		UgUriPart;
-typedef struct UgUriFull_		UgUriFull;
+typedef struct UgUriPart		UgUriPart;
+typedef struct UgUriFull		UgUriFull;
 
 #define	UG_URI_PART_MEMBERS		\
 	const char*		authority;	\
@@ -76,7 +76,7 @@ typedef struct UgUriFull_		UgUriFull;
 // UgUriPart: UgUriPart use smaller stack size than UgUriFull.
 //            It can NOT parse user, password, host, and port in authority.
 //
-struct UgUriPart_
+struct UgUriPart
 {
 	UG_URI_PART_MEMBERS;
 //	const char*		authority;
@@ -96,7 +96,7 @@ int		ug_uri_part_referrer (UgUriPart* upart, const char*  uri);
 // ----------------------------------------------------------------------------
 // UgUriFull: UgUriFull is based on UgUriPart. It can parse authority.
 //
-struct UgUriFull_
+struct UgUriFull
 {
 	UG_URI_PART_MEMBERS;
 //	const char*		authority;
